/******************************************************/
/* class for the compass module (CMPS03) controlling. */
/* file: CompassSensor.h                              */
/* Author: Th. Kaffka, Cologne, Germany               */
/* Date: 04/13/2016                                   */
/******************************************************/

#ifndef CompassSensor_h
#define CompassSensor_h

#include <Wire.h>

// compass parameter. 
#define COMPASS_I2C_BASE_ADR      0b1100000 // The address of the compass (7 bit) (orig: 11000000). 
#define COMPASS_REGISTER_255      0x01
#define COMPASS_REGISTER_3599     0x02
#define COMPASS_REGISTER_FIRMWARE 0x00
#define COMPASS_REGISTER_CALIB    0x0F // calibration register.


class CompassSensor {
  public:
  CompassSensor();
  void initCompass();
  void initEnd();
  int getFirmwareVersion();
  float getCompassData3599();
  int getCompassData255();
};

CompassSensor::CompassSensor() {
}

void CompassSensor::initCompass(void) {
  Wire.beginTransmission(COMPASS_I2C_BASE_ADR);
  Wire.write(COMPASS_REGISTER_CALIB);
  Wire.write(0xFF);
  Wire.endTransmission();
}

void CompassSensor::initEnd(void) {
  Wire.beginTransmission(COMPASS_I2C_BASE_ADR);
  Wire.write(COMPASS_REGISTER_CALIB);
  Wire.write(0x00);
  Wire.endTransmission();
}

int CompassSensor::getFirmwareVersion(void) {
  Wire.beginTransmission(COMPASS_I2C_BASE_ADR);
  Wire.write(COMPASS_REGISTER_FIRMWARE);
  Wire.endTransmission();

  Wire.requestFrom(COMPASS_I2C_BASE_ADR, 1);    // request 1 byte from slave device

  // receive reading from sensor
  int reading = 0;
  if (1 <= Wire.available()) { // if two bytes were received
    reading = Wire.read();  // receive high byte
  }

  return reading;
}

float CompassSensor::getCompassData3599(void) {
  Wire.beginTransmission(COMPASS_I2C_BASE_ADR);
  Wire.write(COMPASS_REGISTER_3599);
  Wire.endTransmission();

  Wire.requestFrom(COMPASS_I2C_BASE_ADR, 2);    // request 2 bytes from slave device

  // receive reading from sensor
  int reading = 0;
  if (2 <= Wire.available()) { // if two bytes were received
    reading = Wire.read();  // receive high byte
    reading = reading << 8;    // shift high byte to be high 8 bits
    reading |= Wire.read(); // receive low byte as lower 8 bits
  }

  return (float) reading / 10.0;
} 

int CompassSensor::getCompassData255(void) {
  Wire.beginTransmission(COMPASS_I2C_BASE_ADR);
  Wire.write(COMPASS_REGISTER_255);
  Wire.endTransmission();

  Wire.requestFrom(COMPASS_I2C_BASE_ADR, 1);    // request 1 byte from slave device

  // receive reading from sensor
  int reading = 0;
  if (1 <= Wire.available()) { // if two bytes were received
    reading = Wire.read();  // receive high byte
  }

  return reading;
} 

#endif