/**********************************************/
/* class for the gear motor.                  */
/* file: GearMotor.h                          */
/* Author: Th. Kaffka, Cologne, Germany       */
/* Date: 02/28/2016                           */
/**********************************************/

#ifndef GearMotor_h
#define GearMotor_h

#include "Arduino.h"

volatile int countA = 0;
volatile int countB = 0;

void encoderCountA() {
  countA++;
}

void encoderCountB() {
  countB++;
}

void resetEncoderCountA() {
  countA = 0;
}

void resetEncoderCountB() {
  countB = 0;
}

int getEncoderCountA() {
  return countA;
}

int getEncoderCountB() {
  return countB;
}

class GearMotor {

  private:
  int pmw;
  int in1;
  int in2;
  int encoder;
  boolean is_moving;

  public:
  GearMotor(int ppmw, int pin1, int pin2, int pencoder=0);
  void resetEncoderCount(); 
  int getEncoderCount(); 
  void forward();
  void backward();
  void setSpeed(int speed);
  void stop();
  boolean isMoving();
};

GearMotor::GearMotor(int ppmw, int pin1, int pin2, int pencoder) {
  pmw = ppmw;
  in1 = pin1;
  in2 = pin2;
  encoder = pencoder;
  is_moving = false;

  pinMode(pmw, OUTPUT);
  pinMode(in1, OUTPUT);
  pinMode(in2, OUTPUT);
  if (encoder == 2) {
	attachInterrupt(digitalPinToInterrupt(encoder), encoderCountA, CHANGE);
  } else if (encoder == 3) {
	attachInterrupt(digitalPinToInterrupt(encoder), encoderCountB, CHANGE);
  }
}

void GearMotor::forward() {
  digitalWrite(in1, HIGH);
  digitalWrite(in2, LOW);
  is_moving = true;
}

void GearMotor::backward() {
  digitalWrite(in1, LOW);
  digitalWrite(in2, HIGH);
  is_moving = true;
}

void GearMotor::setSpeed(int speed) {
  analogWrite(pmw, speed);
}

void GearMotor::stop() {
  digitalWrite(in1, HIGH);
  digitalWrite(in2, HIGH);
  is_moving = false;
}

boolean GearMotor::isMoving() {
  return is_moving;
}

#endif