/************************************************************/
/* class for the lcd display (EA T123W-I2C) controlling.    */
/* file: LCDDisplay.h                                       */
/* Author: Th. Kaffka, Cologne, Germany                     */
/* Date: 03/11/2016                                         */
/************************************************************/

#ifndef LCDDisplay_h
#define LCDDisplay_h

#include <Wire.h>

#define I2C_LCD_ADDR 0b0111010 // 7 Bit adress (orig: 0b01110100).


class LCDDisplay {

  private:
  void textLinePosLCD(byte text_line, byte line_pos);
  char translateLCDText(char text);

  public:
  LCDDisplay();
  void initLCD();
  void clearLCD();
  void textLineOutLCD(char text[], byte text_line, byte text_pos);
  void textOutLCD(char text1[], char text2[], char text3[]);
  void writeIntegerLCD(int16_t number, uint8_t base, byte text_line, byte text_pos);
};

LCDDisplay::LCDDisplay() {
}

void LCDDisplay::initLCD(void) {
  // initialization of display.
  Wire.beginTransmission(I2C_LCD_ADDR);
  Wire.write(0b00000000); // nop
  Wire.write(0b00101110); // function set (DL=0~4-bit, NM=1~4 lines, G=1~voltage generator). 
  Wire.write(0b00001110); // display on, cursor on.
  Wire.write(0b00000110); // increment display.
  Wire.write(0b00000001); // clear.
  Wire.endTransmission();
  delay(1000);
}

void LCDDisplay::clearLCD(void) {
  Wire.beginTransmission(I2C_LCD_ADDR);
  Wire.write(0b00000000); // nop
  Wire.write(0b00000001); // clear.
  Wire.endTransmission();
}

void LCDDisplay::textLinePosLCD(byte text_line, byte line_pos) {
  Wire.beginTransmission(I2C_LCD_ADDR);
  Wire.write(0b00000000); // nop
  if (text_line == 1) {
    Wire.write(0b10000000 + line_pos); // line 1
  } else if (text_line == 2) {
    Wire.write(0b10100000 + line_pos); // line 2
  } else if (text_line == 3) {
    Wire.write(0b11000000 + line_pos); // line 3
  }
  Wire.endTransmission();
}

char LCDDisplay::translateLCDText(char text) {

  if (text >= 97 && text <= 122) { // a .. z
    return text - 97 + 225;
  }

  if (text >= 65 && text <= 90) { // A .. Z
    return text - 65 + 193;
  }

  if (text >= 32 && text <= 64) { // ' ' .. @
    if (text == 36) { // $
      return 130;
    }
    if (text == 64) { // @
      return 128;
    }
    return text - 32 + 160;
  }

  if (text == '[') {
    return 105;
  } else if (text == '\\') {
    return 30;
  } else if (text == ']') {
    return 106;
  } else if (text == '^') {
    return 144;
  } else if (text == '_') {
    return 21;
  } else if (text == '{') {
    return 47;
  } else if (text == '|') {
    return 28;
  } else if (text == '}') {
    return 46;
  } else if (text == '~') {
    return 29;
  }

  return 0;
}

void LCDDisplay::textLineOutLCD(char text[], byte text_line, byte text_pos) {
  textLinePosLCD(text_line, text_pos);

  Wire.beginTransmission(I2C_LCD_ADDR);
  Wire.write(0b01000000); 

  int ndx = 0;
  while (text[ndx] && ndx < 13) {
    Wire.write(translateLCDText(text[ndx])); 
    ndx++;
  }
  Wire.endTransmission();
}

void LCDDisplay::textOutLCD(char text1[], char text2[], char text3[]) {
  textLineOutLCD(text1, 1, 0);
  textLineOutLCD(text2, 2, 0);
  textLineOutLCD(text3, 3, 0);
}

void LCDDisplay::writeIntegerLCD(int16_t number, uint8_t base, byte text_line, byte text_pos) {
  char buffer[12];
  itoa(number, &buffer[0], base);
  textLineOutLCD(buffer, text_line, text_pos);
}

#endif