/**********************************************/
/* class for the vehicle.                     */
/* file: Vehicle.h                            */
/* Author: Th. Kaffka, Cologne, Germany       */
/* Date: 03/16/2016                           */
/**********************************************/

#ifndef Vehicle_h
#define Vehicle_h

#include "Arduino.h"
#include <C:\_Thomas\Ae\C\Roboter\Interface\GearMotor.h>  // adjust this path.

class Vehicle {

  private:
  GearMotor* motor_a;
  GearMotor* motor_b;
  float turn_ratio;
  float degree_per_distance;
  float distance_per_degree;
  void doMove(int move);

  public:
  Vehicle(float wheel_diameter, float track_size, void *pmotor_a, void *pmotor_b);
  boolean isMoving();
  void setSpeed(int speed);
  void forward(int fwd);
  void backward(int bwd);
  void rotateOnly(int dir);
  void rotate(float angle);
  void stop();
};

Vehicle::Vehicle(float wheel_diameter, float track_size, void *pmotor_a, void *pmotor_b) {
  motor_a = (GearMotor *)pmotor_a;
  motor_b = (GearMotor *)pmotor_b;
  turn_ratio = track_size / wheel_diameter;
  degree_per_distance = 24.0 / (3.1415926 * wheel_diameter);
  distance_per_degree = (track_size * 3.1415926) / 360.0;
}

boolean Vehicle::isMoving() {
  return motor_a->isMoving() | motor_b->isMoving();
}

void Vehicle::setSpeed(int speed) {
  motor_a->setSpeed(speed);
  motor_b->setSpeed(speed);
}

// fwd in zentimeter.
void Vehicle::forward(int fwd) {
  motor_a->forward();
  motor_b->forward();
  doMove(fwd);
}

// bwd in zentimeter.
void Vehicle::backward(int bwd) {
  motor_a->backward();
  motor_b->backward();
  doMove(bwd);
}

// -dir -> turn left; dir -> turn right.
void Vehicle::rotateOnly(int dir) {
  if (dir == 1) {
    motor_a->forward();
    motor_b->backward();
  } else if (dir == -1) {
    motor_a->backward();
    motor_b->forward();
  }
}

// -angle -> turn left; angle -> turn right.
void Vehicle::rotate(float angle) {
  int turn = abs(angle) * distance_per_degree;
  if (angle > 0) {
    motor_a->forward();
    motor_b->backward();
  } else if (angle < 0) {
    motor_a->backward();
    motor_b->forward();
  }
  doMove(turn);
}

void Vehicle::stop() {
  motor_a->stop();
  motor_b->stop();
}

void Vehicle::doMove(int move) {
  float target = abs(move) * degree_per_distance;
  if (move != 0) {
	resetEncoderCountA();
	resetEncoderCountB();

	while (motor_a->isMoving() || motor_b->isMoving()) {
	  if (target <= getEncoderCountA()) {
		motor_a->stop();
	  }
	  if (target <= getEncoderCountB()) {
		motor_b->stop();
	  }
	}
  }
}

#endif