/*********************************************/
/* Program moves to the cardinal directions. */
/* file: CardinalDirection.ino               */
/* Author: Th. Kaffka, Cologne, Germany      */
/* Date: 04/15/2016                          */
/*********************************************/

#include <Wire.h>
#include <C:\_Thomas\Ae\C\Roboter\Interface\LCDDisplay.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\Led.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\sound.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\TouchSensor.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\GearMotor.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\Vehicle.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\CompassSensor.h> // adjust this path.

const int SOUND_PIN = 11;

const int PMWA = 5;
const int AIN1 = 8;
const int AIN2 = 7;
const int ENCA = 2;

const int PMWB = 6;
const int BIN1 = 9;
const int BIN2 = 10;
const int ENCB = 3;

GearMotor motor_a(PMWA, AIN1, AIN2, ENCA); 
GearMotor motor_b(PMWB, BIN1, BIN2, ENCB);
Vehicle vehicle(4.2f, 17.5f, &motor_a, &motor_b);

LCDDisplay disp = LCDDisplay();
TouchSensor touch = TouchSensor(12);
Led led = Led(13);
CompassSensor comp = CompassSensor();

boolean start;
boolean led_on;

void setup() {
  Wire.begin();
  disp.initLCD();
  disp.textOutLCD("############", "############", "############");
  delay(1000);
  disp.clearLCD();
  delay(1000);
  disp.textLineOutLCD("CardDir", 1, 0);

  start = true;
  led_on = false;

  vehicle.setSpeed(45); 

  beepMessageReady(SOUND_PIN);
}

void loop() {
  // Bot start/stop statement.
  if (start && touch.isTouch()) {
    start = false;
    led.ledOff();
    beepMessageOk(SOUND_PIN);
    delay(3000);
    vehicle.rotateOnly(1);
    return;
  } else if (start) {
    led_on = !led_on;
    if (led_on) {
      led.ledOn();
    } else {
      led.ledOff();
    }
    delay(500);
    return;
  } else if (!start && touch.isTouch()) {
    vehicle.stop();
    start = true;
    beepMessageOk(SOUND_PIN);
    return;
  }

  float dir = comp.getCompassData3599();

  disp.textLineOutLCD("   ", 2, 0);

  if (dir >= 355.0 or dir <= 5.0) {
    doCardDir((int) dir);
  } else if (dir >= 85.0 and dir <= 95.0) {
    doCardDir((int) dir);
  } else if (dir >= 175.0 and dir <= 185.0) {
    doCardDir((int) dir);
  } else if (dir >= 265.0 and dir <= 275.0) {
    doCardDir((int) dir);
  }

}

void doCardDir(int dir) {
    vehicle.stop();
    disp.writeIntegerLCD(dir, 10, 2, 0);
    delay(3000);
    vehicle.rotateOnly(1);
    delay(300);
}

