/***************************************************************/
/* Program for a Robot which searches the bigges Light source. */
/* file: LightSearch.ino                                       */
/* Author: Th. Kaffka, Cologne, Germany                        */
/* Date: 03/18/2016                                            */
/***************************************************************/

#include <Wire.h>
#include <C:\_Thomas\Ae\C\Roboter\Interface\LCDDisplay.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\Led.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\sound.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\TouchSensor.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\GearMotor.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\Vehicle.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\PhotoTransistor.h> // adjust this path.

const int SOUND_PIN = 11;

const int PMWA = 5;
const int AIN1 = 7;
const int AIN2 = 8;
const int ENCA = 2;

const int PMWB = 6;
const int BIN1 = 9;
const int BIN2 = 10;
const int ENCB = 3;

const int STEP = 12;

GearMotor motor_a(PMWA, AIN1, AIN2, ENCA); 
GearMotor motor_b(PMWB, BIN1, BIN2, ENCB);
Vehicle vehicle(4.2f, 17.5f, &motor_a, &motor_b);

LCDDisplay disp = LCDDisplay();
TouchSensor touch = TouchSensor(12);
Led led = Led(13);
TouchSensor bumper_left = TouchSensor(14);
TouchSensor bumper_right = TouchSensor(15);
PhotoTransistor light = PhotoTransistor(2);

boolean start;
boolean led_on;
int light_value[STEP];
float angle = 360.0 / STEP;

void setup() {
  Wire.begin();
  disp.initLCD();
  disp.textOutLCD("############", "############", "############");
  delay(500);
  disp.clearLCD();
  delay(500);
  disp.textLineOutLCD("LightSearch", 1, 0);

  start = true;
  led_on = false;

  beepMessageReady(SOUND_PIN);
}

void loop() {
  // Bot start/stop statement.
  if (start && touch.isTouch()) {
    start = false;
    led.ledOff();
    beepMessageOk(SOUND_PIN);
    delay(3000);
    return;
  } else if (start) {
    led_on = !led_on;
    if (led_on) {
      led.ledOn();
    } else {
      led.ledOff();
    }
    delay(500);
    return;
  }

  // identify all light sources.
  vehicle.setSpeed(40);
  disp.textLineOutLCD("identify    ", 2, 0);
  for (int i=0; i<STEP; i++) {
    delay(100);
    light_value[i] = getLight();
    vehicle.rotate(angle);
  }
  delay(1000);
  
  // identify the max. light source.
  int max_light = -1000;
  int max_ndx = -1;
  for (int i=0; i<STEP; i++) {
    if (light_value[i] > max_light) {
      max_light = light_value[i];
      max_ndx = i;
    }
  }

  // turn to the max. light source.
  disp.textLineOutLCD("turn to     ", 2, 0);
  for (int i=0; i<max_ndx; i++) {
    delay(100);
    vehicle.rotate(angle);
  }

  disp.textLineOutLCD("go...       ", 2, 0);
  beepMessageAknowledge(SOUND_PIN);
  delay(1000);

  vehicle.setSpeed(50);
  vehicle.forward(0);
  unsigned long limit = millis() + 2000;
  while (limit > millis()) {
    // obstacle => max. lightsource found.
    if (bumper_left.isTouch() || bumper_right.isTouch()) {
      vehicle.stop();
      start = true;
      disp.textLineOutLCD("found       ", 2, 0);
      beepMessageEnd(SOUND_PIN);
      delay(1000);
      return;
    }
  }
}

int getLight() {
  int value = light.readValue();
  disp.textLineOutLCD("    ", 3, 0);
  disp.writeIntegerLCD(value, 10, 3, 0);
  return value;
}

