/***************************************************************/
/* Program avoid obstacles with use of an ultrasonic sensor.   */
/* file: ObstacleAvoid.ino                                     */
/* Author: Th. Kaffka, Cologne, Germany                        */
/* Date: 09/03/2016                                            */
/***************************************************************/

#include <C:\_Thomas\Ae\C\Roboter\Interface\Led.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\sound.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\TouchSensor.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\GearMotor.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\Vehicle.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\UltrasonicSensor.h> // adjust this path.

const int SOUND_PIN = 11;

const int PMWA = 5;
const int AIN1 = 7;
const int AIN2 = 8;
const int ENCA = 2;

const int PMWB = 6;
const int BIN1 = 9;
const int BIN2 = 10;
const int ENCB = 3;

const int TRIG_PIN = 16;
const int ECHO_PIN = 17;

GearMotor motor_a(PMWA, AIN1, AIN2, ENCA); 
GearMotor motor_b(PMWB, BIN1, BIN2, ENCB);
Vehicle vehicle(4.2f, 17.5f, &motor_a, &motor_b);

TouchSensor touch = TouchSensor(12);
Led led = Led(13);
UltrasonicSensor us(TRIG_PIN, ECHO_PIN);

boolean start;
boolean led_on;

void setup() {
  start = true;
  led_on = false;
  beepMessageReady(SOUND_PIN);
}

void loop() {
  // Bot start/stop statement.
  if (start && touch.isTouch()) {
    start = false;
    led.ledOff();
    beepMessageOk(SOUND_PIN);
    delay(3000);
    vehicle.forward(0);
    return;
  } else if (start) {
    led_on = !led_on;
    if (led_on) {
      led.ledOn();
    } else {
      led.ledOff();
    }
    delay(500);
    return;
  }

  if (us.ranging() < 15.0f) {
    vehicle.stop();
    delay(500);
    long distance = random(5, 15);
    vehicle.backward(distance);
    delay(500);
    long angle = random(0, 270) - 135;
    vehicle.rotate(angle);
    delay(500);
    vehicle.forward(0);
  }

}

