/*********************************************/
/* Program moves the platonian areas.        */
/* file: Platon.ino                          */
/* Author: Th. Kaffka, Cologne, Germany      */
/* Date: 03/15/2016                          */
/*********************************************/

#include <Wire.h>
#include <C:\_Thomas\Ae\C\Roboter\Interface\LCDDisplay.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\Led.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\sound.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\TouchSensor.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\GearMotor.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\Vehicle.h> // adjust this path.

const int TRIANGLE = 3;
const int QUARDER  = 4;
const int PENTAGON = 5;

const int SOUND_PIN = 11;

const int PMWA = 5;
const int AIN1 = 7;
const int AIN2 = 8;
const int ENCA = 2;

const int PMWB = 6;
const int BIN1 = 9;
const int BIN2 = 10;
const int ENCB = 3;

GearMotor motor_a(PMWA, AIN1, AIN2, ENCA); 
GearMotor motor_b(PMWB, BIN1, BIN2, ENCB);
Vehicle vehicle(4.2f, 17.5f, &motor_a, &motor_b);

LCDDisplay disp = LCDDisplay();
TouchSensor touch = TouchSensor(12);
Led led = Led(13);

boolean start;
boolean led_on;
int figure = 0; // start with no figure.
int count = 0;
int angle = 0;
int distance = 0;

void setup() {
  Wire.begin();
  disp.initLCD();
  disp.textOutLCD("############", "############", "############");
  delay(1000);
  disp.clearLCD();
  delay(1000);
  disp.textLineOutLCD("Platon", 1, 0);

  start = true;
  led_on = false;

  vehicle.setSpeed(50); 

  beepMessageReady(SOUND_PIN);
}

void loop() {
  // Bot start/stop statement.
  if (start && touch.isTouch()) {
    start = false;
    led.ledOff();
    beepMessageOk(SOUND_PIN);
    delay(3000);
    return;
  } else if (start) {
    led_on = !led_on;
    if (led_on) {
      led.ledOn();
    } else {
      led.ledOff();
    }
    delay(500);
    return;
  }

  if (count == 0) {
        
    beepMessageReady(SOUND_PIN);

    if (figure == 0) {
      figure = TRIANGLE;
      disp.textLineOutLCD("--triangle--", 2, 0);
    } else if (figure == TRIANGLE) {
      figure = QUARDER;
      disp.textLineOutLCD("--quarder---", 2, 0);
    } else if (figure == QUARDER) {
      figure = PENTAGON;
      disp.textLineOutLCD("--pentagon--", 2, 0);
    } else {
      disp.textLineOutLCD("----end-----", 2, 0);
      start = true;
      figure = 0;
      return;
    }
        
    count = figure;
    angle = 360 / figure;
    distance = 180 / figure;
  }
      
  vehicle.forward(distance);
  vehicle.rotate(angle);
  count--;
}
