/*********************************************/
/* Program for a Robot with TouchSensors.    */
/* file: Touch.ino                           */
/* Author: Th. Kaffka, Cologne, Germany      */
/* Date: 03/18/2016                          */
/*********************************************/

#include <Wire.h>
#include <C:\_Thomas\Ae\C\Roboter\Interface\LCDDisplay.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\Led.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\sound.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\TouchSensor.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\GearMotor.h> // adjust this path.
#include <C:\_Thomas\Ae\C\Roboter\Interface\Vehicle.h> // adjust this path.

const int SOUND_PIN = 11;

const int PMWA = 5;
const int AIN1 = 7;
const int AIN2 = 8;
const int ENCA = 2;

const int PMWB = 6;
const int BIN1 = 9;
const int BIN2 = 10;
const int ENCB = 3;

GearMotor motor_a(PMWA, AIN1, AIN2, ENCA); 
GearMotor motor_b(PMWB, BIN1, BIN2, ENCB);
Vehicle vehicle(4.2f, 17.5f, &motor_a, &motor_b);

LCDDisplay disp = LCDDisplay();
TouchSensor touch = TouchSensor(12);
Led led = Led(13);
TouchSensor bumper_left = TouchSensor(14);
TouchSensor bumper_right = TouchSensor(15);

boolean start;
boolean led_on;

void setup() {
  Wire.begin();
  disp.initLCD();
  disp.textOutLCD("############", "############", "############");
  delay(500);
  disp.clearLCD();
  delay(500);
  disp.textLineOutLCD("Touch", 1, 0);

  start = true;
  led_on = false;
  randomSeed(analogRead(2));

  vehicle.setSpeed(40);

  beepMessageReady(SOUND_PIN);
}

void loop() {
  // Bot start/stop statement.
  if (start && touch.isTouch()) {
    start = false;
    led.ledOff();
    beepMessageOk(SOUND_PIN);
    delay(3000);
    vehicle.forward(0);
    return;
  } else if (start) {
    led_on = !led_on;
    if (led_on) {
      led.ledOn();
    } else {
      led.ledOff();
    }
    delay(500);
    return;
  }

  if (bumper_left.isTouch() || bumper_right.isTouch()) {
    if (bumper_left.isTouch()) {
      disp.textLineOutLCD("bumper left ", 2, 0);
    } else {
      disp.textLineOutLCD("bumper right", 2, 0);
    }
    vehicle.stop();
    delay(500);
    long distance = random(5, 15);
    vehicle.backward(distance);
    delay(500);
    long angle = random(0, 270) - 135;
    vehicle.rotate(angle);
    delay(500);
    vehicle.forward(0);
    disp.textLineOutLCD("            ", 2, 0);
  }

}
