/**********************************************/
/* class for the gear motor.                  */
/* file: GearMotor.h                          */
/* Author: Th. Kaffka, Cologne, Germany       */
/* Date: 02/28/2016                           */
/**********************************************/

#ifndef GearMotor_h
#define GearMotor_h

#include "Arduino.h"

class GearMotor {
  private:
  int pmw;
  int in1;
  int in2;
  int encoder;
  int count;
  byte count_status;

  public:
  GearMotor(int ppmw, int pin1, int pin2, int pencoder=0);
  void resetEncoderCount(); 
  int getEncoderCount(); 
  void forward();
  void backward();
  void setSpeed(int speed);
  void stop();
  void encoderCount();
};

GearMotor::GearMotor(int ppmw, int pin1, int pin2, int pencoder) {
  pmw = ppmw;
  in1 = pin1;
  in2 = pin2;
  encoder = pencoder;
  count = 0;
  count_status = false;

  pinMode(pmw, OUTPUT);
  pinMode(in1, OUTPUT);
  pinMode(in2, OUTPUT);
  if (encoder != 0) {
    pinMode(encoder, INPUT);
    digitalWrite(encoder, HIGH); // activate the pull up resistor. 
  }

}

void GearMotor::resetEncoderCount() {
  count = 0;
}

int GearMotor::getEncoderCount() {
  return count;
}

void GearMotor::forward() {
  digitalWrite(in1, HIGH);
  digitalWrite(in2, LOW);
}

void GearMotor::backward() {
  digitalWrite(in1, LOW);
  digitalWrite(in2, HIGH);
}

void GearMotor::setSpeed(int speed) {
  analogWrite(pmw, speed);
}

void GearMotor::stop() {
  digitalWrite(in1, HIGH);
  digitalWrite(in2, HIGH);
}

void GearMotor::encoderCount() {
  if (encoder == 0) return;
  int data = digitalRead(encoder);
  if (!count_status && (data == HIGH)) {
    count_status = true;
    count++;
  }
  if (count_status && (data == LOW)) {
    count_status = false;
    count++;
  }
}

#endif
