#!/usr/bin/python3
import sys
sys.path.append("/home/pi/Ae/Python/Roboter/lib")
import spidev
import RPi.GPIO as GPIO
import time
import tool4picture as t4pic

###################################################################
# Display Functions (for the ST7735)                              #
# file: LCDST7735.py (Version 0.1)                                #
# Author: Th. Kaffka, Cologne, Germany                            #
# Date: 03/10/2016                                                #
# Credits: Bruce E. Hall, Programs from 04/27/2013 and 02/19/2014 #
#          Adafruit Industries, Prorgam from 2013                 #
#          Michael Kofler et al. 'Raspberry Pi'                   #
###################################################################

######################################################### 
# pin connection of display ST7735 with Raspberry PI 2. #
# Display       GPIO(pin)                               #
# VCC           1    (3,3 V)                            #
# GND           9    (0 V)                              #
# SCL           23   (SCLK)                             #
# SDA           19   (MOSI)                             #
# RS/DC         18   (GPIO5)                            #
# RES           22   (GPIO6)                            #
# CS            24   (CE0)                              #
#########################################################

##################################################################
# Necessary installations.                                       #
# 1. Call the 'raspi-config' with 'sudo'. Then                   #
#    choose under the menu point 'Advanced Options' the point    #
#    'SPI'. Answer the questions with 'yes'.                     #
# 2. At next, download the spidev installation from:             #
#    'https://github.com/Gadgetoid/py-spidev'.                   #
# 3. Unpack the installation in a folder. Goto the folder in     #
#    a LXTerminal Window and enter the install command:          #
#    'sudo python3 setup.py install' and reboot:                 #
#    'sudo shutdown -r now'                                      #
# 4. If you get the message, that Python.h can not be found,     #
#    install the package python-dev with:                        #
#    'sudo apt-get install python3-dev' for phython 3. Use the   #
#    name python-dev for phyton 2.                               #
##################################################################

# color definitions.
BLACK   = 0x000000
RED     = 0xFF0000
GREEN   = 0x00FF00
BLUE    = 0x0000FF
MAROON  = 0x800000
LIME    = 0x008000
NAVY    = 0x000080
OLIVE   = 0x808000
PURPLE  = 0x800080
TEAL    = 0x008080
GRAY    = 0x808080
SILVER  = 0x0C0C0C
GOLD    = 0xFFD700
ORANGE  = 0xFFA500
YELLOW  = 0xFFFF00
FUCHSIA = 0xFF00FF
AQUA    = 0x00FFFF
WHITE   = 0xFFFFFF

font = [0x00, 0x00, 0x00, 0x00, 0x00,
	0x3E, 0x5B, 0x4F, 0x5B, 0x3E,
	0x3E, 0x6B, 0x4F, 0x6B, 0x3E,
	0x1C, 0x3E, 0x7C, 0x3E, 0x1C,
	0x18, 0x3C, 0x7E, 0x3C, 0x18,
	0x1C, 0x57, 0x7D, 0x57, 0x1C,
	0x1C, 0x5E, 0x7F, 0x5E, 0x1C,
	0x00, 0x18, 0x3C, 0x18, 0x00,
	0xFF, 0xE7, 0xC3, 0xE7, 0xFF,
	0x00, 0x18, 0x24, 0x18, 0x00,
	0xFF, 0xE7, 0xDB, 0xE7, 0xFF,
	0x30, 0x48, 0x3A, 0x06, 0x0E,
	0x26, 0x29, 0x79, 0x29, 0x26,
	0x40, 0x7F, 0x05, 0x05, 0x07,
	0x40, 0x7F, 0x05, 0x25, 0x3F,
	0x5A, 0x3C, 0xE7, 0x3C, 0x5A,
	0x7F, 0x3E, 0x1C, 0x1C, 0x08,
	0x08, 0x1C, 0x1C, 0x3E, 0x7F,
	0x14, 0x22, 0x7F, 0x22, 0x14,
	0x5F, 0x5F, 0x00, 0x5F, 0x5F,
	0x06, 0x09, 0x7F, 0x01, 0x7F,
	0x00, 0x66, 0x89, 0x95, 0x6A,
	0x60, 0x60, 0x60, 0x60, 0x60,
	0x94, 0xA2, 0xFF, 0xA2, 0x94,
	0x08, 0x04, 0x7E, 0x04, 0x08,
	0x10, 0x20, 0x7E, 0x20, 0x10,
	0x08, 0x08, 0x2A, 0x1C, 0x08,
	0x08, 0x1C, 0x2A, 0x08, 0x08,
	0x1E, 0x10, 0x10, 0x10, 0x10,
	0x0C, 0x1E, 0x0C, 0x1E, 0x0C,
	0x30, 0x38, 0x3E, 0x38, 0x30,
	0x06, 0x0E, 0x3E, 0x0E, 0x06,
	0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x5F, 0x00, 0x00,
	0x00, 0x07, 0x00, 0x07, 0x00,
	0x14, 0x7F, 0x14, 0x7F, 0x14,
	0x24, 0x2A, 0x7F, 0x2A, 0x12,
	0x23, 0x13, 0x08, 0x64, 0x62,
	0x36, 0x49, 0x56, 0x20, 0x50,
	0x00, 0x08, 0x07, 0x03, 0x00,
	0x00, 0x1C, 0x22, 0x41, 0x00,
	0x00, 0x41, 0x22, 0x1C, 0x00,
	0x2A, 0x1C, 0x7F, 0x1C, 0x2A,
	0x08, 0x08, 0x3E, 0x08, 0x08,
	0x00, 0x80, 0x70, 0x30, 0x00,
	0x08, 0x08, 0x08, 0x08, 0x08,
	0x00, 0x00, 0x60, 0x60, 0x00,
	0x20, 0x10, 0x08, 0x04, 0x02,
	0x3E, 0x51, 0x49, 0x45, 0x3E,
	0x00, 0x42, 0x7F, 0x40, 0x00,
	0x72, 0x49, 0x49, 0x49, 0x46,
	0x21, 0x41, 0x49, 0x4D, 0x33,
	0x18, 0x14, 0x12, 0x7F, 0x10,
	0x27, 0x45, 0x45, 0x45, 0x39,
	0x3C, 0x4A, 0x49, 0x49, 0x31,
	0x41, 0x21, 0x11, 0x09, 0x07,
	0x36, 0x49, 0x49, 0x49, 0x36,
	0x46, 0x49, 0x49, 0x29, 0x1E,
	0x00, 0x00, 0x14, 0x00, 0x00,
	0x00, 0x40, 0x34, 0x00, 0x00,
	0x00, 0x08, 0x14, 0x22, 0x41,
	0x14, 0x14, 0x14, 0x14, 0x14,
	0x00, 0x41, 0x22, 0x14, 0x08,
	0x02, 0x01, 0x59, 0x09, 0x06,
	0x3E, 0x41, 0x5D, 0x59, 0x4E,
	0x7C, 0x12, 0x11, 0x12, 0x7C,
	0x7F, 0x49, 0x49, 0x49, 0x36,
	0x3E, 0x41, 0x41, 0x41, 0x22,
	0x7F, 0x41, 0x41, 0x41, 0x3E,
	0x7F, 0x49, 0x49, 0x49, 0x41,
	0x7F, 0x09, 0x09, 0x09, 0x01,
	0x3E, 0x41, 0x41, 0x51, 0x73,
	0x7F, 0x08, 0x08, 0x08, 0x7F,
	0x00, 0x41, 0x7F, 0x41, 0x00,
	0x20, 0x40, 0x41, 0x3F, 0x01,
	0x7F, 0x08, 0x14, 0x22, 0x41,
	0x7F, 0x40, 0x40, 0x40, 0x40,
	0x7F, 0x02, 0x1C, 0x02, 0x7F,
	0x7F, 0x04, 0x08, 0x10, 0x7F,
	0x3E, 0x41, 0x41, 0x41, 0x3E,
	0x7F, 0x09, 0x09, 0x09, 0x06,
	0x3E, 0x41, 0x51, 0x21, 0x5E,
	0x7F, 0x09, 0x19, 0x29, 0x46,
	0x26, 0x49, 0x49, 0x49, 0x32,
	0x03, 0x01, 0x7F, 0x01, 0x03,
	0x3F, 0x40, 0x40, 0x40, 0x3F,
	0x1F, 0x20, 0x40, 0x20, 0x1F,
	0x3F, 0x40, 0x38, 0x40, 0x3F,
	0x63, 0x14, 0x08, 0x14, 0x63,
	0x03, 0x04, 0x78, 0x04, 0x03,
	0x61, 0x59, 0x49, 0x4D, 0x43,
	0x00, 0x7F, 0x41, 0x41, 0x41,
	0x02, 0x04, 0x08, 0x10, 0x20,
	0x00, 0x41, 0x41, 0x41, 0x7F,
	0x04, 0x02, 0x01, 0x02, 0x04,
	0x40, 0x40, 0x40, 0x40, 0x40,
	0x00, 0x03, 0x07, 0x08, 0x00,
	0x20, 0x54, 0x54, 0x78, 0x40,
	0x7F, 0x28, 0x44, 0x44, 0x38,
	0x38, 0x44, 0x44, 0x44, 0x28,
	0x38, 0x44, 0x44, 0x28, 0x7F,
	0x38, 0x54, 0x54, 0x54, 0x18,
	0x00, 0x08, 0x7E, 0x09, 0x02,
	0x18, 0xA4, 0xA4, 0x9C, 0x78,
	0x7F, 0x08, 0x04, 0x04, 0x78,
	0x00, 0x44, 0x7D, 0x40, 0x00,
	0x20, 0x40, 0x40, 0x3D, 0x00,
	0x7F, 0x10, 0x28, 0x44, 0x00,
	0x00, 0x41, 0x7F, 0x40, 0x00,
	0x7C, 0x04, 0x78, 0x04, 0x78,
	0x7C, 0x08, 0x04, 0x04, 0x78,
	0x38, 0x44, 0x44, 0x44, 0x38,
	0xFC, 0x18, 0x24, 0x24, 0x18,
	0x18, 0x24, 0x24, 0x18, 0xFC,
	0x7C, 0x08, 0x04, 0x04, 0x08,
	0x48, 0x54, 0x54, 0x54, 0x24,
	0x04, 0x04, 0x3F, 0x44, 0x24,
	0x3C, 0x40, 0x40, 0x20, 0x7C,
	0x1C, 0x20, 0x40, 0x20, 0x1C,
	0x3C, 0x40, 0x30, 0x40, 0x3C,
	0x44, 0x28, 0x10, 0x28, 0x44,
	0x4C, 0x90, 0x90, 0x90, 0x7C,
	0x44, 0x64, 0x54, 0x4C, 0x44,
	0x00, 0x08, 0x36, 0x41, 0x00,
	0x00, 0x00, 0x77, 0x00, 0x00,
	0x00, 0x41, 0x36, 0x08, 0x00,
	0x02, 0x01, 0x02, 0x04, 0x02,
	0x3C, 0x26, 0x23, 0x26, 0x3C,
	0x1E, 0xA1, 0xA1, 0x61, 0x12,
	0x3A, 0x40, 0x40, 0x20, 0x7A,
	0x38, 0x54, 0x54, 0x55, 0x59,
	0x21, 0x55, 0x55, 0x79, 0x41,
	0x22, 0x54, 0x54, 0x78, 0x42,
	0x21, 0x55, 0x54, 0x78, 0x40,
	0x20, 0x54, 0x55, 0x79, 0x40,
	0x0C, 0x1E, 0x52, 0x72, 0x12,
	0x39, 0x55, 0x55, 0x55, 0x59,
	0x39, 0x54, 0x54, 0x54, 0x59,
	0x39, 0x55, 0x54, 0x54, 0x58,
	0x00, 0x00, 0x45, 0x7C, 0x41,
	0x00, 0x02, 0x45, 0x7D, 0x42,
	0x00, 0x01, 0x45, 0x7C, 0x40,
	0x7D, 0x12, 0x11, 0x12, 0x7D,
	0xF0, 0x28, 0x25, 0x28, 0xF0,
	0x7C, 0x54, 0x55, 0x45, 0x00,
	0x20, 0x54, 0x54, 0x7C, 0x54,
	0x7C, 0x0A, 0x09, 0x7F, 0x49,
	0x32, 0x49, 0x49, 0x49, 0x32,
	0x3A, 0x44, 0x44, 0x44, 0x3A,
	0x32, 0x4A, 0x48, 0x48, 0x30,
	0x3A, 0x41, 0x41, 0x21, 0x7A,
	0x3A, 0x42, 0x40, 0x20, 0x78,
	0x00, 0x9D, 0xA0, 0xA0, 0x7D,
	0x3D, 0x42, 0x42, 0x42, 0x3D,
	0x3D, 0x40, 0x40, 0x40, 0x3D,
	0x3C, 0x24, 0xFF, 0x24, 0x24,
	0x48, 0x7E, 0x49, 0x43, 0x66,
	0x2B, 0x2F, 0xFC, 0x2F, 0x2B,
	0xFF, 0x09, 0x29, 0xF6, 0x20,
	0xC0, 0x88, 0x7E, 0x09, 0x03,
	0x20, 0x54, 0x54, 0x79, 0x41,
	0x00, 0x00, 0x44, 0x7D, 0x41,
	0x30, 0x48, 0x48, 0x4A, 0x32,
	0x38, 0x40, 0x40, 0x22, 0x7A,
	0x00, 0x7A, 0x0A, 0x0A, 0x72,
	0x7D, 0x0D, 0x19, 0x31, 0x7D,
	0x26, 0x29, 0x29, 0x2F, 0x28,
	0x26, 0x29, 0x29, 0x29, 0x26,
	0x30, 0x48, 0x4D, 0x40, 0x20,
	0x38, 0x08, 0x08, 0x08, 0x08,
	0x08, 0x08, 0x08, 0x08, 0x38,
	0x2F, 0x10, 0xC8, 0xAC, 0xBA,
	0x2F, 0x10, 0x28, 0x34, 0xFA,
	0x00, 0x00, 0x7B, 0x00, 0x00,
	0x08, 0x14, 0x2A, 0x14, 0x22,
	0x22, 0x14, 0x2A, 0x14, 0x08,
	0x55, 0x00, 0x55, 0x00, 0x55,
	0xAA, 0x55, 0xAA, 0x55, 0xAA,
	0xFF, 0x55, 0xFF, 0x55, 0xFF,
	0x00, 0x00, 0x00, 0xFF, 0x00,
	0x10, 0x10, 0x10, 0xFF, 0x00,
	0x14, 0x14, 0x14, 0xFF, 0x00,
	0x10, 0x10, 0xFF, 0x00, 0xFF,
	0x10, 0x10, 0xF0, 0x10, 0xF0,
	0x14, 0x14, 0x14, 0xFC, 0x00,
	0x14, 0x14, 0xF7, 0x00, 0xFF,
	0x00, 0x00, 0xFF, 0x00, 0xFF,
	0x14, 0x14, 0xF4, 0x04, 0xFC,
	0x14, 0x14, 0x17, 0x10, 0x1F,
	0x10, 0x10, 0x1F, 0x10, 0x1F,
	0x14, 0x14, 0x14, 0x1F, 0x00,
	0x10, 0x10, 0x10, 0xF0, 0x00,
	0x00, 0x00, 0x00, 0x1F, 0x10,
	0x10, 0x10, 0x10, 0x1F, 0x10,
	0x10, 0x10, 0x10, 0xF0, 0x10,
	0x00, 0x00, 0x00, 0xFF, 0x10,
	0x10, 0x10, 0x10, 0x10, 0x10,
	0x10, 0x10, 0x10, 0xFF, 0x10,
	0x00, 0x00, 0x00, 0xFF, 0x14,
	0x00, 0x00, 0xFF, 0x00, 0xFF,
	0x00, 0x00, 0x1F, 0x10, 0x17,
	0x00, 0x00, 0xFC, 0x04, 0xF4,
	0x14, 0x14, 0x17, 0x10, 0x17,
	0x14, 0x14, 0xF4, 0x04, 0xF4,
	0x00, 0x00, 0xFF, 0x00, 0xF7,
	0x14, 0x14, 0x14, 0x14, 0x14,
	0x14, 0x14, 0xF7, 0x00, 0xF7,
	0x14, 0x14, 0x14, 0x17, 0x14,
	0x10, 0x10, 0x1F, 0x10, 0x1F,
	0x14, 0x14, 0x14, 0xF4, 0x14,
	0x10, 0x10, 0xF0, 0x10, 0xF0,
	0x00, 0x00, 0x1F, 0x10, 0x1F,
	0x00, 0x00, 0x00, 0x1F, 0x14,
	0x00, 0x00, 0x00, 0xFC, 0x14,
	0x00, 0x00, 0xF0, 0x10, 0xF0,
	0x10, 0x10, 0xFF, 0x10, 0xFF,
	0x14, 0x14, 0x14, 0xFF, 0x14,
	0x10, 0x10, 0x10, 0x1F, 0x00,
	0x00, 0x00, 0x00, 0xF0, 0x10,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xF0, 0xF0, 0xF0, 0xF0, 0xF0,
	0xFF, 0xFF, 0xFF, 0x00, 0x00,
	0x00, 0x00, 0x00, 0xFF, 0xFF,
	0x0F, 0x0F, 0x0F, 0x0F, 0x0F,
	0x38, 0x44, 0x44, 0x38, 0x44,
	0xFC, 0x4A, 0x4A, 0x4A, 0x34,
	0x7E, 0x02, 0x02, 0x06, 0x06,
	0x02, 0x7E, 0x02, 0x7E, 0x02,
	0x63, 0x55, 0x49, 0x41, 0x63,
	0x38, 0x44, 0x44, 0x3C, 0x04,
	0x40, 0x7E, 0x20, 0x1E, 0x20,
	0x06, 0x02, 0x7E, 0x02, 0x02,
	0x99, 0xA5, 0xE7, 0xA5, 0x99,
	0x1C, 0x2A, 0x49, 0x2A, 0x1C,
	0x4C, 0x72, 0x01, 0x72, 0x4C,
	0x30, 0x4A, 0x4D, 0x4D, 0x30,
	0x30, 0x48, 0x78, 0x48, 0x30,
	0xBC, 0x62, 0x5A, 0x46, 0x3D,
	0x3E, 0x49, 0x49, 0x49, 0x00,
	0x7E, 0x01, 0x01, 0x01, 0x7E,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x44, 0x44, 0x5F, 0x44, 0x44,
	0x40, 0x51, 0x4A, 0x44, 0x40,
	0x40, 0x44, 0x4A, 0x51, 0x40,
	0x00, 0x00, 0xFF, 0x01, 0x03,
	0xE0, 0x80, 0xFF, 0x00, 0x00,
	0x08, 0x08, 0x6B, 0x6B, 0x08,
	0x36, 0x12, 0x36, 0x24, 0x36,
	0x06, 0x0F, 0x09, 0x0F, 0x06,
	0x00, 0x00, 0x18, 0x18, 0x00,
	0x00, 0x00, 0x10, 0x10, 0x00,
	0x30, 0x40, 0xFF, 0x01, 0x01,
	0x00, 0x1F, 0x01, 0x01, 0x1E,
	0x00, 0x19, 0x1D, 0x17, 0x12,
	0x00, 0x3C, 0x3C, 0x3C, 0x3C,
	0x00, 0x00, 0x00, 0x00, 0x00]

class LCDST7735:

   # ST7735 command definitions.
   SWRESET = 0x01
   SLPOUT  = 0x11
   DISPON  = 0x29
   CASET   = 0x2A
   RASET   = 0x2B
   RAMWR   = 0x2C
   MADCTL  = 0x36
   COLMOD  = 0x3A

   # common constants.
   WIDTH = 127
   HEIGHT = 159

   spi = None
   cursor_x = 0
   cursor_y = 0
   textsize = 1
   textcolor = BLACK
   textbgcolor = WHITE
   wrap = True

   def __init__(self, dc, res):
      self.dc = dc
      self.res = res
      GPIO.setmode(GPIO.BOARD)
      GPIO.setwarnings(False)
      GPIO.setup(dc, GPIO.OUT)
      GPIO.setup(res, GPIO.OUT, initial=GPIO.HIGH)
      self.spi = spidev.SpiDev()
#      self.spi.mode = 0
      self.spi.open(0, 0) # use the CE0 pin for 'chip select'.
      self.hardwareReset()
      self.initDisplay()

   def __del__(self):
      self.spi.close()
      GPIO.cleanup()

   #################
   # hardware reset.
   def hardwareReset(self):
      GPIO.output(self.res, GPIO.LOW)
      GPIO.output(self.res, GPIO.HIGH)

   ########################
   # initalize the display.
   def initDisplay(self):
      self.writeCmd(self.SWRESET)
      time.sleep(0.2)
      self.writeCmd(self.SLPOUT)
      time.sleep(0.2)
      self.writeCmd(self.DISPON)
      time.sleep(0.2)

   #############
   # set cursor.
   def setCursor(self, x, y):
      self.cursor_x = x
      self.cursor_y = y

   ###########################
   # get cursors x coordinate.
   def getCursorX(self):
      return self.cursor_x

   ###########################
   # get cursors y coordinate.
   def getCursorY(self):
      return self.cursor_y

   #######################
   # set size of the text.
   def setTextSize(self, s):
      if (s > 0):
         self.textsize = s
      else:
         self.textsize = 1

   ########################
   # set color of the text.
   def setTextColorBoth(self, c):
      self.textcolor = c
      self.textbgcolor = c

   ########################
   # set color of the text.
   def setTextColor(self, c, b):
      self.textcolor = c
      self.textbgcolor = b

   ###########
   # set wrap.
   def setTextWrap(self, w):
      self.wrap = w

   ###############################
   # send a command to the ST7735.
   def writeCmd(self, cmd):
      GPIO.output(self.dc, GPIO.LOW)
      self.spi.writebytes([cmd])   

   ##########################
   # send data to the ST7735.
   def writeData(self, data):
      GPIO.output(self.dc, GPIO.HIGH)
      self.spi.writebytes([data])

   #####################################
   # send a word (16 bit) to the ST7735.
   def writeWord(self, data):
      self.writeData(data >> 8)
      self.writeData(data & 0xFF)

   ################
   # write a color.  
   def writeColor(self, color, width, count):
      red = color >> 16
      green = (color >> 8) & 0xFF
      blue = color & 0xFF
      rgb = [red, green, blue]
      GPIO.output(self.dc, GPIO.HIGH)
      for i in range (count):
         self.spi.writebytes(rgb * width)

   ##################
   # define a window.
   def setAddrWindow(self, x0, y0, x1, y1):
      self.writeCmd(self.CASET)
      self.writeWord(x0)
      self.writeWord(x1)
      self.writeCmd(self.RASET)
      self.writeWord(y0)
      self.writeWord(y1)

   ###############
   # draw a pixel.
   def drawPixel(self, x, y, color):
      self.setAddrWindow(x, y, x, y)
      self.writeCmd(self.RAMWR)
      self.writeColor(color, 1, 1)

   ##############
   # draw a char.
   def drawChar(self, x, y, c, color, bg, size):

      if((x >= self.WIDTH) \
         or (y >= self.HEIGHT) \
         or ((x + 6 * size - 1) < 0) \
         or ((y + 8 * size - 1) < 0)): 
         return

      if(c >= 176):
         c += 1; # Handle 'classic' charset behavior

      for i in range (0, 6):
      
         line = 0
         if (i == 5):  
            line = 0x0
         else: 
            line = font[(c*5)+i]

         for j in range (0, 8):
            if (line & 0x1):
               if (size == 1): # default size
                  self.drawPixel(x+i, y+j, color)
               else:  # big size
                  self.fillRect2(x+(i*size), y+(j*size), size, size, color)
            elif (bg != color):
               if (size == 1): # default size
                  self.drawPixel(x+i, y+j, bg)
               else: # big size
                  self.fillRect2(x+i*size, y+j*size, size, size, bg)
            line = line >> 1

   ################
   # draw a string.
   def drawString(self, text):
      for char in text:
         if (char == '\n'):
            self.cursor_y += self.textsize * 8
            self.cursor_x = 0
         elif (char == '\r'):
            pass 
         else:
            self.drawChar(self.cursor_x, self.cursor_y, ord(char), self.textcolor, self.textbgcolor, self.textsize)
            self.cursor_x += self.textsize * 6
            if (self.wrap and (self.cursor_x > (self.WIDTH - self.textsize * 6))):
               self.cursor_y += self.textsize * 8
               self.cursor_x = 0

   ##################
   # draw an integer.
   def drawInt(self, number):
      self.drawString(str(number))

   ############################
   # draw a fast vertical line.
   def drawVLine(self, x, y, h, color):
      self.drawLine(x, y, x, y+h-1, color)

   ##############################
   # draw a fast horizontal line.
   def drawHLine(self, x, y, w, color):
      self.drawLine(x, y, x+w-1, y, color)

   ##############
   # draw a line.
   def drawLine(self, x0, y0, x1, y1, color):

      steep = abs(y1 - y0) > abs(x1 - x0)
      if (steep):
         temp = x0; x0 = y0; y0 = temp
         temp = x1; x1 = y1; y1 = temp

      if (x0 > x1):
         temp = x0; x0 = x1; x1 = temp
         temp = y0; y0 = y1; y1 = temp

      dx = x1 - x0
      dy = abs(y1 - y0)

      err = dx // 2

      if (y0 < y1):
         ystep = 1
      else:
         ystep = -1

      for i in range(x0, x1+1):
         if (steep):
            self.drawPixel(y0, i, color)
         else:
            self.drawPixel(i, y0, color)

         err -= dy
         if (err < 0):
            y0 += ystep
            err += dx

   ########################
   # draw a circle outline.
   def drawCircle(self, x0, y0, r, color):
      f = 1 - r
      ddF_x = 1
      ddF_y = -2 * r
      x = 0
      y = r

      self.drawPixel(x0  , y0+r, color)
      self.drawPixel(x0  , y0-r, color)
      self.drawPixel(x0+r, y0  , color)
      self.drawPixel(x0-r, y0  , color)

      while (x<y):
         if (f >= 0):
            y -= 1
            ddF_y += 2
            f += ddF_y
         x += 1
         ddF_x += 2
         f += ddF_x;
  
         self.drawPixel(x0 + x, y0 + y, color)
         self.drawPixel(x0 - x, y0 + y, color)
         self.drawPixel(x0 + x, y0 - y, color)
         self.drawPixel(x0 - x, y0 - y, color)
         self.drawPixel(x0 + y, y0 + x, color)
         self.drawPixel(x0 - y, y0 + x, color)
         self.drawPixel(x0 + y, y0 - x, color)
         self.drawPixel(x0 - y, y0 - x, color)

   ##################################
   # draw a circle (helper function).
   def drawCircleHelper(self, x0, y0, r, cornername, color):
      f = 1 - r
      ddF_x = 1
      ddF_y = -2 * r
      x = 0
      y = r

      while (x < y):
         if (f >= 0):
            y -= 1
            ddF_y += 2
            f += ddF_y
         x += 1
         ddF_x += 2
         f += ddF_x
         if (cornername & 0x4):
            self.drawPixel(x0 + x, y0 + y, color)
            self.drawPixel(x0 + y, y0 + x, color)
         if (cornername & 0x2):
            self.drawPixel(x0 + x, y0 - y, color)
            self.drawPixel(x0 + y, y0 - x, color)
         if (cornername & 0x8):
            self.drawPixel(x0 - y, y0 + x, color)
            self.drawPixel(x0 - x, y0 + y, color)
         if (cornername & 0x1):
            self.drawPixel(x0 - y, y0 - x, color)
            self.drawPixel(x0 - x, y0 - y, color)

   ################
   # fill a circle.
   def fillCircle(self, x0, y0, r, color):
      self.drawVLine(x0, y0-r, 2*r+1, color)
      self.fillCircleHelper(x0, y0, r, 3, 0, color)

   ##################################
   # fill a circle (helper function).
   def fillCircleHelper(self, x0, y0, r, cornername, delta, color):
      f = 1 - r
      ddF_x = 1
      ddF_y = -2 * r
      x = 0
      y = r

      while (x<y):
         if (f >= 0):
            y -= 1
            ddF_y += 2
            f += ddF_y
         x += 1
         ddF_x += 2
         f += ddF_x

         if (cornername & 0x1):
            self.drawVLine(x0+x, y0-y, 2*y+1+delta, color)
            self.drawVLine(x0+y, y0-x, 2*x+1+delta, color)
         if (cornername & 0x2):
            self.drawVLine(x0-x, y0-y, 2*y+1+delta, color)
            self.drawVLine(x0-y, y0-x, 2*x+1+delta, color)

   ###################
   # draw a rectangle.
   def drawRect(self, x, y, w, h, color):
      self.drawHLine(x, y, w, color)
      self.drawHLine(x, y+h-1, w, color)
      self.drawVLine(x, y, h, color)
      self.drawVLine(x+w-1, y, h, color)

   ###################
   # fill a rectangle.
   def fillRect(self, x0, y0, x1, y1, color):
      width = x1 - x0 + 1
      height = y1 - y0 + 1
      self.setAddrWindow(x0, y0, x1, y1)
      self.writeCmd(self.RAMWR)
      self.writeColor(color, width, height)

   ###################
   # fill a rectangle.
   def fillRect2(self, x, y, w, h, color):
      for i in range (x, x+w):
         self.drawVLine(i, y, h, color);

   ###################
   # fill a rectangle.
   def fillRect3(self, x0, y0, x1, y1, color):
      w = x1 - x0
      h = y1 - y0
      for i in range (x0, x0+w):
         self.drawVLine(i, y0, h, color);

   ###########################
   # draw a rounded rectangle.
   def drawRoundRect(self, x, y, w, h, r, color):
      self.drawHLine(x+r  , y    , w-2*r, color)
      self.drawHLine(x+r  , y+h-1, w-2*r, color)
      self.drawVLine(x    , y+r  , h-2*r, color)
      self.drawVLine(x+w-1, y+r  , h-2*r, color)
      # draw four corners
      self.drawCircleHelper(x+r    , y+r    , r, 1, color);
      self.drawCircleHelper(x+w-r-1, y+r    , r, 2, color);
      self.drawCircleHelper(x+w-r-1, y+h-r-1, r, 4, color);
      self.drawCircleHelper(x+r    , y+h-r-1, r, 8, color);

   ###########################
   # fill a rounded rectangle.
   def fillRoundRect(self, x, y, w, h, r, color):
      self.fillRect(x+r, y, w-2*r, h, color)
      # draw four corner.
      self.fillCircleHelper(x+w-r-1, y+r, r, 1, h-2*r-1, color)
      self.fillCircleHelper(x+r    , y+r, r, 2, h-2*r-1, color)

   #############################################
   # fill the whole display screen with a color.
   def fillScreen(self, color):
      self.fillRect(0, 0, self.WIDTH, self.HEIGHT, color)

   #####################################################
   # fill the whole display screen with the color black.
   def clearScreen(self):
      self.fillRect(0, 0, self.WIDTH, self.HEIGHT, BLACK)

   ################
   # draw a bitmap.
   def drawBitMap(self, bmp, x, y, w, h):
      header = t4pic.getBitmapHeader(bmp)
      width = header[2]
      height = header[3]
      if (width > w): width = w
      if (height > h): height = h

      for row in range(height-1, -1, -1):
         for col in range(0, width):
            pixel = t4pic.getPixel(height - row - 1, col, bmp, header)
            pixel_color = pixel[0] << 16
            pixel_color = pixel_color | (pixel[1] << 8)
            pixel_color = pixel_color | pixel[2]
            self.drawPixel(col + x, row + y, pixel_color)
   
