#!/usr/bin/python3
import sys
import RPi.GPIO as GPIO
import time

#########################################
# Class to make sonds with the speaker. #
# file: PiezoSpeaker.py                 #
# Author: Th. Kaffka, Cologne, Germany  #
# Date: 02/09/2016                      # 
#########################################

TONE_B5 = 988
TONE_A5 = 880
TONE_G5 = 784
TONE_F5 = 698
TONE_E5 = 659
TONE_D5 = 587
TONE_C5 = 523
TONE_B4 = 494
TONE_A4 = 440
TONE_G4 = 392
TONE_F4 = 349
TONE_E4 = 330
TONE_D4 = 294
TONE_C4 = 262
TONE_B3 = 247
TONE_A3 = 220
TONE_G3 = 196
TONE_F3 = 175
TONE_E3 = 165
TONE_D3 = 145
TONE_C3 = 131

class PiezoSpeaker:

   def __init__(self, pin_number):
      self.pin_number = pin_number
      GPIO.setmode(GPIO.BOARD)
      GPIO.setup(self.pin_number, GPIO.OUT)
      self.speaker = GPIO.PWM(self.pin_number, 100)
      self.speaker.start(0)

   def __del__(self):
      self.speaker.ChangeDutyCycle(0)

   def sound(self, note, duration):
      self.speaker.ChangeDutyCycle(50)
      self.speaker.ChangeFrequency(note)
      time.sleep(duration)
      self.speaker.ChangeDutyCycle(0)

   def beepMessageReady(self):
      self.sound(TONE_C4, 0.1)
      self.sound(TONE_F4, 0.2)
      self.sound(TONE_E4, 0.1)
      self.sound(TONE_G4, 0.3)

   def beepMessageEnd(self):
      self.sound(TONE_G4, 0.1)
      self.sound(TONE_A4, 0.25)
      time.sleep(0.05)
      self.sound(TONE_A4, 0.25)
      self.sound(TONE_E4, 0.3)

   def beepMessageError(self):
      self.sound(TONE_A3, 0.4)
      self.sound(TONE_F3, 0.25)
      self.sound(TONE_A3, 0.8)

   def beepMessageOk(self):
      self.sound(TONE_C5, 0.2)
      self.sound(TONE_F5, 0.35)
      self.sound(TONE_C5, 0.15)

   def beepMessageAknowledge(self):
      self.sound(TONE_G5, 0.3)
      self.sound(TONE_C5, 0.2)

   def beepMessageDataout(self):
      self.sound(TONE_G5, 0.15)
      time.sleep(0.15)
      self.sound(TONE_G5, 0.15)
      self.sound(TONE_C5, 0.3)

