#!/usr/bin/python3
import sys
sys.path.append("/home/pi/Ae/Python/Roboter/Interface")
import RPi.GPIO as GPIO
import time
from GearMotor import GearMotor

####################################################
# Program to move a vehicle.                       #
# file: Vehicle.py                                 #
# Author: Th. Kaffka, Cologne, Germany             #
# Date: 04/04/2016                                 #
####################################################

class Vehicle:

   def __init__(self, pweel_diameter, ptrack_size, pmotor_a, pmotor_b):
      self.weel_diameter = pweel_diameter
      self.track_size = ptrack_size
      self.motor_a = pmotor_a
      self.motor_b = pmotor_b
      self.turn_ratio = self.track_size / self.weel_diameter
      self.degree_per_distance = 24.0 / (3.1415926 * self.weel_diameter)
      self.distance_per_degree = (self.track_size * 3.1415926) / 360.0

   def __del__(self):
      pass 

   def isMoving(self):
      return self.motor_a.isMoving() or self.motor_b.isMoving()

   def setSpeed(self, speed):
      self.motor_a.setSpeed(speed)
      self.motor_b.setSpeed(speed)

   def forward(self, fwd=0):
      self.motor_a.resetEncoderCount()
      self.motor_b.resetEncoderCount()
      self.motor_a.forward()
      self.motor_b.forward()
      self.doMove(fwd)
   
   def backward(self, bwd=0):
      self.motor_a.resetEncoderCount()
      self.motor_b.resetEncoderCount()
      self.motor_a.backward()
      self.motor_b.backward()
      self.doMove(bwd)

   def rotate(self, angle):
      turn = abs(angle) * self.distance_per_degree
      self.motor_a.resetEncoderCount()
      self.motor_b.resetEncoderCount()
      if (angle > 0):
         self.motor_a.forward()
         self.motor_b.backward()
      elif (angle < 0):
         self.motor_a.backward()
         self.motor_b.forward()
      self.doMove(turn)

   def stop(self):
      self.motor_a.stop()
      self.motor_b.stop()

   def doMove(self, move):
      target = (int) (abs(move) * self.degree_per_distance) - 1
      if (not move == 0):
         while(self.motor_a.isMoving() or self.motor_b.isMoving()):
            if (target <= self.motor_a.getEncoderCount()):
               self.motor_a.stop()
            if (target <= self.motor_b.getEncoderCount()):
               self.motor_b.stop()
            time.sleep(0.1)
            
         
