#!/usr/bin/python3
import sys
sys.path.append("/home/pi/Ae/Python/Roboter/Interface")
import RPi.GPIO as GPIO
import time
from GearMotor import GearMotor
from Vehicle import Vehicle
from Led import Led
from TouchSensor import TouchSensor
from PiezoSpeaker import PiezoSpeaker
from UltrasonicSensor import UltrasonicSensor
from random import *

####################################################
# Program keeps on the desktop.                    #
# file: DeskTop.py                                 #
# Author: Th. Kaffka, Cologne, Germany             #
# Date: 05/24/2016                                 #
####################################################

SPEED_FWD = 20
SPEED_BAK = 27
SPEED_ROT = 35

PMWA = 40 
AIN2 = 38
AIN1 = 36
ENCA = 32

PMWB = 37 
BIN1 = 35
BIN2 = 33
ENCB = 31

motor_a = GearMotor(PMWA, AIN2, AIN1, ENCA)
motor_b = GearMotor(PMWB, BIN1, BIN2, ENCB)
vehicle = Vehicle(4.2, 17.5, motor_a, motor_b)

button = TouchSensor(11)
led = Led(13)
speaker = PiezoSpeaker(15)
bumper_left = TouchSensor(16)
bumper_right = TouchSensor(18)
bumper_back = TouchSensor(12)

TRIG_PIN = 29
ECHO_PIN = 7
ussensor = UltrasonicSensor(TRIG_PIN, ECHO_PIN)

start = True
led_on = False

speaker.beepMessageReady()

####################
# avoid an obstacle.
def avoid():
   vehicle.stop()
   time.sleep(0.5)
   end = time.time() + uniform(0.5, 1.5)
   vehicle.setSpeed(SPEED_BAK)
   vehicle.backward()
   while (end > time.time()):
      if (bumper_back.isTouch()):
         vehicle.stop()
         time.sleep(0.5)
         vehicle.forward(7)
         break
   
   vehicle.stop()
   time.sleep(0.5)
   angle = randint(0, 270) - 135
   vehicle.setSpeed(SPEED_ROT)
   vehicle.rotate(angle)
   time.sleep(0.5)
   vehicle.setSpeed(SPEED_FWD)
   vehicle.forward()

try:

   while(True):
      if (start and button.isTouch()):
         start = False
         led.ledOff()
         speaker.beepMessageOk()
         time.sleep(3)
         vehicle.setSpeed(SPEED_FWD)
         vehicle.forward()
         continue
      elif(start):
         led_on = not led_on
         if (led_on):
            led.ledOn()
         else:
            led.ledOff()
         time.sleep(0.5)
         continue
      elif ((not start) and button.isTouch()):
         vehicle.stop()
         start = True
         speaker.beepMessageOk()
         continue

      if (bumper_left.isTouch() or bumper_right.isTouch()):
         avoid()
      elif (ussensor.distance() > 7.0):
         avoid()

except KeyboardInterrupt:
   pass
finally:
   GPIO.cleanup()


