#!/usr/bin/python3
import sys
sys.path.append("/home/pi/Ae/Python/Roboter/Interface")
import RPi.GPIO as GPIO
import time
import os
from GearMotor import GearMotor
from Led import Led
from TouchSensor import TouchSensor
from PiezoSpeaker import PiezoSpeaker

####################################################
# Program controlls a dinner tray by ssh.          #
# file: DinnerTraySSH.py                           #
# Author: Th. Kaffka, Cologne, Germany             #
# Date: 06/25/2016                                 #
####################################################

SPEED_FWD = 18 # motor speed forward.

PMWA = 40 
AIN2 = 38
AIN1 = 36
ENCA = 32

PMWB = 37 
BIN1 = 35
BIN2 = 33
ENCB = 31

motor_left = GearMotor(PMWA, AIN2, AIN1, ENCA)
motor_right = GearMotor(PMWB, BIN1, BIN2, ENCB)

button = TouchSensor(11)
led = Led(13)
speaker = PiezoSpeaker(15)

STOP = 0
GO = 1

LEFT = 0
STRAIGHT = 1
RIGHT = 2

start = True
led_on = False

onoff = STOP
leftright = STRAIGHT

speaker.beepMessageReady()

try:

   while(True):
      if (start and button.isTouch()):
         start = False
         led.ledOff()
         speaker.beepMessageOk()
         time.sleep(3)
         continue
      elif(start):
         led_on = not led_on
         if (led_on):
            led.ledOn()
         else:
            led.ledOff()
         time.sleep(0.5)
         continue
      elif ((not start) and button.isTouch()):
         start = True
         f.close()
         motor_left.stop()
         motor_right.stop()
         speaker.beepMessageOk()
         continue

      # look for command on / off
      try:
         f = open ("./cmd/onoff.txt", "r")
         if (onoff == STOP):
            onoff = GO
         else:
            onoff = STOP
         f.close()
         os.remove("./cmd/onoff.txt")
      except FileNotFoundError:
         pass

      # look for command left / straight / right
      try:
         f = open ("./cmd/leftright.txt", "r")
         if (leftright == LEFT):
            leftright = STRAIGHT
         elif (leftright == STRAIGHT):
            leftright = RIGHT
         else:
            leftright = LEFT
         f.close()
         os.remove("./cmd/leftright.txt")
      except FileNotFoundError:
         pass

      # steer the robot.
      if (onoff == STOP):
         motor_left.stop()
         motor_right.stop()
      elif (onoff == GO):
         motor_left.setSpeed(SPEED_FWD)
         motor_right.setSpeed(SPEED_FWD)

      if (leftright == LEFT and onoff == GO):
         motor_left.backward()
         motor_right.forward()
      elif (leftright == STRAIGHT and onoff == GO):
         motor_left.forward()
         motor_right.forward()
      elif (leftright == RIGHT and onoff == GO):
         motor_left.forward()
         motor_right.backward()

      time.sleep(0.1)

except KeyboardInterrupt:
   pass
finally:
   motor_left.stop()
   motor_right.stop()
   GPIO.cleanup()


