#!/usr/bin/python3
import sys
sys.path.append("/home/pi/Ae/Python/Roboter/Interface")
import random
import time
from LCDST7735 import *

##################################################
# Program test of the display ST7735.            #
# file: LCDTestST7735.py (Version 0.1)           #
# Author: Th. Kaffka, Cologne, Germany           #
# Date: 03/10/2016                               #
##################################################

# pin definitions.
DC = 18 # pin for sending data / command.
RES = 22 # pin for hardware reset.
disp = LCDST7735(DC, RES)

errCode = 0
errMess = ""

######################
# loads a bitmap file.
def loadPicture(file_name):
   global errCode
   global errMess
   errCode = 0

   try:
      f = open ("./data/" + file_name, "rb")
      img = f.read()
      f.close()
      return img
   except FileNotFoundError:
      errCode = 1
      errMess = "File " + file_name + " not found!"
   except:
      print (traceback.print_exc())

# hello screen.
disp.clearScreen()
disp.setCursor(0, 24)
disp.setTextColorBoth(WHITE)
disp.setTextSize(2)
disp.setTextWrap(True)
disp.drawString("Graphic on a ST7735, controlled by a Raspberry Pi 2.")
time.sleep(6)

# screen with colors.
disp.fillScreen(RED)
disp.fillScreen(GREEN)
disp.fillScreen(BLUE)

# screen with geometrical figures.
disp.fillScreen(BLUE)
disp.drawCircle(80, 80, 20, GREEN)
disp.drawHLine(10, 10, 100, GREEN)
disp.drawVLine(10, 10, 100, GREEN)
disp.drawRect(50, 50, 40, 40, GREEN)
disp.fillCircle(10, 100, 14, GREEN)
disp.drawRoundRect(50, 100, 14, 20, 7, GREEN)
disp.fillRoundRect(100, 100, 14, 20, 7, GREEN)

time.sleep(5)

# screen with a picture.
img = loadPicture("DSC00307_2.bmp")
if (errCode != 0):
   print(errMess)
else:
   disp.drawBitMap(img, 0, 0, disp.WIDTH+1, disp.HEIGHT+1)

time.sleep(5)

# screen with lines.
disp.fillScreen(BLUE)
for y in range (0, disp.HEIGHT, 5):
   disp.drawLine(0, 0, disp.WIDTH, y, GREEN)
for y in range (0, disp.HEIGHT, 5):
   disp.drawLine(0, disp.HEIGHT, disp.WIDTH, y, RED)

time.sleep(5)

# screen with characters.
disp.fillScreen(GREEN)
disp.setCursor(0, 0)
disp.setTextColorBoth(BLACK)
disp.setTextSize(1)
disp.drawString("Habe nun, ach! Philosophie, Juristerei und Medizin, Und leider auch Theologie Durchaus studiert, mit heissem Bemuehn. " \
                + "Da steh ich nun, ich armer Tor! Und bin so klug als wie zuvor; " \
                + "Heisse Magister, heisse Doktor gar Und ziehe schon an die zehen Jahr Herauf, herab und quer und krumm Meine Schueler an der Nase herum - " \
                + "Und sehe, dass wir nichts wissen koennen! Das will mir schier das Herz verbrennen.")

time.sleep(10)

# screen with random points.
disp.clearScreen()

for i in range(0, 10000):
   x = random.randrange(0, 128)
   y = random.randrange(0, 160)
   color_1 = random.randrange(0, 260)
   color_2 = random.randrange(0, 260)
   color_3 = random.randrange(0, 260)
   color = color_1 << 16
   color = color | (color_2 << 8)
   color = color | color_3
   disp.drawPixel(x, y, color)

# screen with random colors.
for i in range(0, 10):
   color_1 = random.randrange(0, 260)
   color_2 = random.randrange(0, 260)
   color_3 = random.randrange(0, 260)
   color = color_1 << 16
   color = color | (color_2 << 8)
   color = color | color_3
   disp.fillScreen(color)

# the end screen.
disp.setCursor(0, 50)
disp.setTextColorBoth(WHITE)
disp.setTextSize(3)
disp.drawString("The End")
time.sleep(3)

disp.clearScreen()

# close the spi and the GPIO interface.
del disp
