#!/usr/bin/python3
import sys
sys.path.append("/home/pi/Ae/Python/Roboter/Interface")
import tkinter as tk
from threading import Thread
import RPi.GPIO as GPIO
import time
# import the interface classes here begin.
from PiezoSpeaker import *
# import the interface classes here end.

########################################
# control program                      #
# file: ControlProgram.pyw             #
# Author: Th. Kaffka, Cologne, Germany #
# Date: 05/01/2016                     #
########################################

# define constants.
PROG_NAME = "Piezo Sound"
PROG_VERSION = "0.1"
PIC_WIDTH = 250
PIC_HEIGHT = 50

# define the hardware interface here begin.
speaker = PiezoSpeaker(40)
# define the hardware interface here end.

# define globals.
processing_go = False

class PiezoSound(tk.Frame):

   def __init__(self, master=None):
      tk.Frame.__init__(self, master)
      self.grid()
      self.createWidgets()
      
   def createWidgets(self):
      """ create the widgets of the surface. """
      self.picture_ground = tk.Canvas(self, height=str(PIC_HEIGHT), width=str(PIC_WIDTH), background="#ADD8E6")
      self.picture_ground.create_text(PIC_WIDTH - 140, PIC_HEIGHT-20, text=PROG_NAME, fill="#00008B", tag="coord")
      self.picture_ground.pack()
      self.startButton = tk.Button(self, text="Start", command=self.startRobot).pack(fill=tk.X)
      self.stopButton = tk.Button(self, text="Stop", command=self.stopRobot).pack(fill=tk.X)
      self.quitButton = tk.Button(self, text="Quit", command=self.winClose).pack(fill=tk.X)
      self.messageLabel = tk.Label(self, text="")
      self.messageLabel.pack(fill=tk.X)

   def winClose(self):
      """ exits the program. """
      if (processing_go): return
      # delete the objects here begin.
      global speaker
      del speaker
      # delete the objects here end.
      GPIO.cleanup()
      self.quit()
      self.destroy()

   def startRobot(self):
      """ starts the robots processing. """
      global processing_go
      self.messageLabel.configure(text = "Start Robot processing.")
      processing_go = True
      self.t = Thread(target=self.loopRobotProcessing, args=())
      self.t.start()

   def stopRobot(self):
      """ stops the robots processing. """
      global processing_go
      if (not processing_go): return
      self.messageLabel.configure(text = "Stop Robot processing.")
      processing_go = False

   def loopRobotProcessing(self):
      """ processes the robot in a loop. """
      while (processing_go):
         # enter the robots controlling statements here begin.
         speaker.beepMessageReady()
         time.sleep(1.0)
         speaker.beepMessageEnd()
         time.sleep(1.0)
         speaker.beepMessageError()
         time.sleep(1.0)
         speaker.beepMessageOk()
         time.sleep(1.0)
         speaker.beepMessageAknowledge()
         time.sleep(1.0)
         speaker.beepMessageDataout()
         time.sleep(1.0)
         speaker.sound(TONE_C4, 0.5)
         speaker.sound(TONE_E4, 0.5)
         speaker.sound(TONE_G4, 0.5)
         speaker.sound(TONE_C5, 0.5)
         time.sleep(1.0)
         # enter the robots controlling statements here end.
         
app = PiezoSound()
app.master.title(PROG_NAME + " (" + PROG_VERSION + ")")
app.mainloop()
