#!/usr/bin/python3
import sys
sys.path.append("/home/pi/Ae/Python/Roboter/Interface")
import RPi.GPIO as GPIO
import time
from random import randint
from GearMotor import GearMotor
from Vehicle import Vehicle
from Led import Led
from TouchSensor import TouchSensor
from PiezoSpeaker import PiezoSpeaker
from LCDST7735 import *
from PhotoTransistor import PhotoTransistor

####################################################
# Program searchs the bigest light source.         #
# file: LightSearch.py                             #
# Author: Th. Kaffka, Cologne, Germany             #
# Date: 04/08/2016                                 #
####################################################

# define common constants.
STEP = 12

# define motor constants.
PMWA = 40 
AIN2 = 38
AIN1 = 36
ENCA = 32

PMWB = 37 
BIN1 = 35
BIN2 = 33
ENCB = 31

# define photo transistor constants.
SPICLK = 3
SPIMISO = 5
SPIMOSI = 16
SPICS = 12

# define the objects.
button = TouchSensor(11)
led = Led(13)
speaker = PiezoSpeaker(15)
bumper_left = TouchSensor(7)
bumper_right = TouchSensor(29)
ptrans = PhotoTransistor(0, SPICLK, SPIMOSI, SPIMISO, SPICS)

motor_a = GearMotor(PMWA, AIN2, AIN1, ENCA)
motor_b = GearMotor(PMWB, BIN1, BIN2, ENCB)
vehicle = Vehicle(4.2, 16.0, motor_a, motor_b)

DC = 18 # pin for sending data / command.
RES = 22 # pin for hardware reset.
disp = LCDST7735(DC, RES)

start = True
led_on = False
light_value = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
angle = 360 / STEP

disp.clearScreen()
disp.setCursor(0, 24)
disp.setTextColorBoth(WHITE)
disp.setTextSize(2)
disp.setTextWrap(False)
disp.drawString("Touch")
disp.fillRect2(0, 40, disp.WIDTH, 16, RED)

speaker.beepMessageReady()

def printLCD(text):
   disp.fillRect2(0, 40, disp.WIDTH, 16, RED)
   disp.setCursor(0, 40)
   disp.drawString(text)

def getLight():
   value = ptrans.readValue()
   return value

try:

   while(True):
      if (start and button.isTouch()):
         start = False
         led.ledOff()
         speaker.beepMessageOk()
         time.sleep(3)
         vehicle.stop()
         vehicle.setSpeed(25)
         continue
      elif(start):
         led_on = not led_on
         if (led_on):
            led.ledOn()
         else:
            led.ledOff()
         time.sleep(0.5)
         continue
      elif ((not start) and button.isTouch()):
         vehicle.stop()
         start = True
         speaker.beepMessageOk()
         continue

      # identify all light sources.
      printLCD("identify...")
      for i in range(0, STEP):
         time.sleep(0.1)
         light_value[i] = getLight()
         vehicle.rotate(angle)
      time.sleep(1)

      # identify the max. light source.
      max_light = -1000
      max_ndx = -1
      for i in range(0, STEP):
         if (light_value[i] > max_light):
            max_light = light_value[i]
            max_ndx = i + 1

      # turn to the max. light source.
      printLCD("turn to...")
      for i in range(0, max_ndx):
         time.sleep(0.1)
         vehicle.rotate(angle)

      # move 2 secs straight.
      printLCD("go...")
      speaker.beepMessageAknowledge()
      time.sleep(1)
      
      vehicle.forward()
      end_time = time.time() + 2
      while (end_time > time.time()):
         if (bumper_left.isTouch() or bumper_right.isTouch()):
            vehicle.stop()
            printLCD("found...")
            start = True
            speaker.beepMessageEnd()
            break

      vehicle.stop()
      time.sleep(1)

except KeyboardInterrupt:
   pass
finally:
   GPIO.cleanup()


