#!/usr/bin/python3
import sys
sys.path.append("/home/pi/Ae/Python/Roboter/Interface")
import RPi.GPIO as GPIO
import time
import os
from GearMotor import GearMotor
from Led import Led
from TouchSensor import TouchSensor
from PiezoSpeaker import PiezoSpeaker
from PhotoTransistor import PhotoTransistor

####################################################
# Program tracks a line.                           #
# file: LineTrackinmg.py                           #
# Author: Th. Kaffka, Cologne, Germany             #
# Date: 06/07/2016                                 #
####################################################

SPEED_FWD = 18         # motor speed forward.
SPEED_MIN = 0          # min. motor speed.
SPEED_ROT = 15         # motor speed for rotation.
LIGHT_INIT_CORR = 3    # light correction for initialization.
CURR_LIGHT_CORR = -10    # light correction for steering.
ADJUSTMENT_FACTOR = 7  # adjustment factor for steering.

PMWA = 40 
AIN2 = 38
AIN1 = 36
ENCA = 32

PMWB = 37 
BIN1 = 35
BIN2 = 33
ENCB = 31

motor_left = GearMotor(PMWA, AIN2, AIN1, ENCA)
motor_right = GearMotor(PMWB, BIN1, BIN2, ENCB)

SPICLK = 12
SPIMISO = 16
SPIMOSI = 18
SPICS = 22

ptrans_left = PhotoTransistor(0, SPICLK, SPIMOSI, SPIMISO, SPICS)
ptrans_right = PhotoTransistor(1, SPICLK, SPIMOSI, SPIMISO, SPICS)

button = TouchSensor(11)
led = Led(13)
speaker = PiezoSpeaker(15)

LEFT = 0
RIGHT = 1

start = True
led_on = False
line_sensor_init = False
light_value_left = []
light_value_right = []
speed_left = SPEED_FWD
speed_right = SPEED_FWD
curr_adjustment_factor = ADJUSTMENT_FACTOR
curr_dir = 2
old_dir = 2

f = open ("./data/ltdata.txt", "w")
write_time = time.time()

speaker.beepMessageReady()

##############################
# initializes the line-sensor.
def initLineSensor(dir):
   motor_left.stop()
   motor_right.stop()
   motor_left.setSpeed(SPEED_ROT)
   motor_right.setSpeed(SPEED_ROT)
   light_value_result = []

   # get the light value for white.
   if (dir == LEFT):
      light_value_result.append(ptrans_left.readValue())
   elif (dir == RIGHT):
      light_value_result.append(ptrans_right.readValue())

   # initialize the light min. search.
   if (dir == LEFT):
      light_value = ptrans_left.readValue()
   elif (dir == RIGHT):
      light_value = ptrans_right.readValue()
   light_value_next = -1000
   light_value_min = 1000

   # rotate.
   if (dir == LEFT): # turn right.
      motor_left.forward()
      motor_right.backward()
   elif (dir == RIGHT): # turn left.
      motor_left.backward()
      motor_right.forward()

   # get the light value for black.
   while (light_value > light_value_next and not button.isTouch()):
      if (dir == LEFT):
         light_value = ptrans_left.readValue()
      elif (dir == RIGHT):
         light_value = ptrans_right.readValue()
         
      if (light_value_min > light_value):
         light_value_min = light_value

      time.sleep(0.01)
         
      if (dir == LEFT):
         light_value_next = ptrans_left.readValue() - LIGHT_INIT_CORR
      elif (dir == RIGHT):
         light_value_next = ptrans_right.readValue() - LIGHT_INIT_CORR

   motor_left.stop()
   motor_right.stop()

   light_value_result.append(light_value_min)

   # calculate the average light value.
   light_value_result.append(light_value_result[1] + ((light_value_result[0] - light_value_result[1]) / 2))

   return light_value_result

#############################
# rises the speed of a motor.
def riseSpeed(speed, adjustment):
   speed = speed + adjustment
   if (speed > SPEED_FWD):
      speed = SPEED_FWD
   return speed

##############################
# lowers the speed of a motor.
def sinkSpeed(speed, adjustment):
   speed = speed - adjustment
   if (speed < SPEED_MIN):
      speed = SPEED_MIN
   return speed

########################
# lowers the adjustment.
def sinkAdjustment(factor):
   factor -= 1
   if (factor < 1):
      factor = 1
   return factor

##########################################
# writes the line tracking data to a file.
def writeToFile(cll, clr, sl, sr):
   global write_time
   if (write_time < time.time()):
      f.write(str(cll) + "," + str(clr) + "," + str(sl) + "," + str(sr) + "\r\n")
      write_time = time.time() + 0.2

try:

   while(True):
      if (start and button.isTouch()):
         start = False
         led.ledOff()
         speaker.beepMessageOk()
         time.sleep(3)
         continue
      elif(start):
         led_on = not led_on
         if (led_on):
            led.ledOn()
         else:
            led.ledOff()
         time.sleep(0.5)
         continue
      elif ((not start) and button.isTouch()):
         start = True
         f.close()
         motor_left.stop()
         motor_right.stop()
         speaker.beepMessageOk()
         continue

      # initialize the line-sensor.
      if (not line_sensor_init):
         light_value_left = initLineSensor(LEFT)
         speaker.beepMessageOk()
         while(not button.isTouch()):
            time.sleep(0.1)
         time.sleep(2)
         light_value_right = initLineSensor(RIGHT)
         speaker.beepMessageOk()
         while(not button.isTouch()):
            time.sleep(0.1)
         speaker.beepMessageReady()
         time.sleep(2)
         motor_left.setSpeed(SPEED_FWD)
         motor_right.setSpeed(SPEED_FWD)
         motor_left.forward()
         motor_right.forward()
         line_sensor_init = True

      # steer the robot.
      curr_light_left = ptrans_left.readValue()
      curr_light_right = ptrans_right.readValue()
      if (curr_light_left < light_value_left[2] - CURR_LIGHT_CORR): # steer left.
         speed_left = sinkSpeed(speed_left, curr_adjustment_factor)
         speed_right = riseSpeed(speed_right, curr_adjustment_factor)
         curr_dir = LEFT
         curr_adjustment_factor = sinkAdjustment(curr_adjustment_factor)
         writeToFile(curr_light_left, curr_light_right, speed_left, speed_right)
      elif (curr_light_right < light_value_right[2] - CURR_LIGHT_CORR): # steer right.
         speed_left = riseSpeed(speed_left, curr_adjustment_factor)
         speed_right = sinkSpeed(speed_right, curr_adjustment_factor)
         curr_dir = RIGHT
         curr_adjustment_factor = sinkAdjustment(curr_adjustment_factor)
         writeToFile(curr_light_left, curr_light_right, speed_left, speed_right)

      if (curr_dir != old_dir):
         curr_adjustment_factor = ADJUSTMENT_FACTOR
         old_dir = curr_dir

      motor_left.setSpeed(speed_left)
      motor_right.setSpeed(speed_right)

      time.sleep(0.001)

except KeyboardInterrupt:
   pass
finally:
   f.close()
   motor_left.stop()
   motor_right.stop()
   GPIO.cleanup()


