#!/usr/bin/python3
import sys
sys.path.append("/home/pi/Ae/Python/Roboter/Interface")
import RPi.GPIO as GPIO
import time
import os
from GearMotor import GearMotor
from Led import Led
from TouchSensor import TouchSensor
from PiezoSpeaker import PiezoSpeaker
from PhotoTransistor import PhotoTransistor
from NeuralNetwork import NeuralNetwork

#######################################################
# Program tracks a line with use of a neural network. #
# file: LineTrackinmgNN.py                            #
# Author: Th. Kaffka, Cologne, Germany                #
# Date: 06/10/2016                                    #
#######################################################

SPEED_FWD = 18 # motor speed forward.

PMWA = 40 
AIN2 = 38
AIN1 = 36
ENCA = 32

PMWB = 37 
BIN1 = 35
BIN2 = 33
ENCB = 31

motor_left = GearMotor(PMWA, AIN2, AIN1, ENCA)
motor_right = GearMotor(PMWB, BIN1, BIN2, ENCB)

SPICLK = 12
SPIMISO = 16
SPIMOSI = 18
SPICS = 22

ptrans_left = PhotoTransistor(0, SPICLK, SPIMOSI, SPIMISO, SPICS)
ptrans_right = PhotoTransistor(1, SPICLK, SPIMOSI, SPIMISO, SPICS)

button = TouchSensor(11)
led = Led(13)
speaker = PiezoSpeaker(15)

# insert your values begin.
factor = 0.010309278350515464
nn = NeuralNetwork(2, 2, 0.3, 6, [], 0, factor)
nn.w1[0][0] = -4.091914864376606
nn.w1[1][0] = 17.97499603321279
nn.w1[2][0] = -0.8685621956846833
nn.w1[0][1] = -2.2087187868611418
nn.w1[1][1] = -0.09388638591545495
nn.w1[2][1] = -0.7187928136586081
nn.w1[0][2] = 20.466806408527678
nn.w1[1][2] = 17.976646220200312
nn.w1[2][2] = -16.824881331574108
nn.w1[0][3] = -19.785952202641713
nn.w1[1][3] = -16.065529711982165
nn.w1[2][3] = 16.853326581549382
nn.w1[0][4] = -8.125646843835996
nn.w1[1][4] = -0.06832278420275452
nn.w1[2][4] = 1.6862684759500228
nn.w1[0][5] = -10.653503954150297
nn.w1[1][5] = 5.715602072847136
nn.w1[2][5] = -2.3574988829772763
nn.w1[0][6] = 0.04601523584942813
nn.w1[1][6] = 0.8047650941213722
nn.w1[2][6] = 0.2620286869860472
nn.w2[0][0] = -2.1081975672995825
nn.w2[1][0] = -2.2826501065933185
nn.w2[2][0] = 4.846586087428946
nn.w2[3][0] = 4.353277039970315
nn.w2[4][0] = 2.1873172404113173
nn.w2[5][0] = -3.131183505902003
nn.w2[6][0] = -5.881468288581799
nn.w2[0][1] = 2.3029640833137837
nn.w2[1][1] = -0.3773773469308533
nn.w2[2][1] = -2.867365821262275
nn.w2[3][1] = -2.499220187059058
nn.w2[4][1] = -2.2621921278195365
nn.w2[5][1] = 4.403078859643815
nn.w2[6][1] = -0.9145902525676322
# insert your values end.

start = True
led_on = False

speaker.beepMessageReady()

try:

   while(True):
      if (start and button.isTouch()):
         start = False
         led.ledOff()
         speaker.beepMessageOk()
         time.sleep(3)
         motor_left.setSpeed(SPEED_FWD)
         motor_right.setSpeed(SPEED_FWD)
         motor_left.forward()
         motor_right.forward()
         continue
      elif(start):
         led_on = not led_on
         if (led_on):
            led.ledOn()
         else:
            led.ledOff()
         time.sleep(0.5)
         continue
      elif ((not start) and button.isTouch()):
         start = True
         motor_left.stop()
         motor_right.stop()
         speaker.beepMessageOk()
         continue

      # steer the robot.
      curr_light_left = ptrans_left.readValue()
      curr_light_right = ptrans_right.readValue()
      result = nn.executeFunction([curr_light_left, curr_light_right])
      speed_left = result[0]
      speed_right = result[1]
      motor_left.setSpeed(speed_left)
      motor_right.setSpeed(speed_right)
      time.sleep(0.001)

except KeyboardInterrupt:
   pass
finally:
   motor_left.stop()
   motor_right.stop()
   GPIO.cleanup()


