#!/usr/bin/python3
import sys
sys.path.append("/home/pi/Ae/Python/Roboter/Interface")
import RPi.GPIO as GPIO
import time
import math
from GearMotor import GearMotor
from Vehicle import Vehicle
from Led import Led
from TouchSensor import TouchSensor
from UltrasonicSensor import UltrasonicSensor
from PiezoSpeaker import PiezoSpeaker
from LCDST7735 import *
from random import randint

#############################################################
# Program avoid obstacles with use of an ultrasonic sensor. #
# file: ObstacleAvoidExercise.py                            #
# Author: Th. Kaffka, Cologne, Germany                      #
# Date: 04/17/2016                                          #
#############################################################

PMWA = 40 
AIN2 = 38
AIN1 = 36
ENCA = 32

PMWB = 37 
BIN1 = 35
BIN2 = 33
ENCB = 31

button = TouchSensor(11)
led = Led(13)
speaker = PiezoSpeaker(15)

TRIG_PIN = 29
ECHO_PIN = 7
ussensor = UltrasonicSensor(TRIG_PIN, ECHO_PIN)
bumper_left = TouchSensor(12)
bumper_right = TouchSensor(16)

motor_a = GearMotor(PMWA, AIN2, AIN1, ENCA)
motor_b = GearMotor(PMWB, BIN1, BIN2, ENCB)
vehicle = Vehicle(4.2, 17.5, motor_a, motor_b)

DC = 18 # pin for sending data / command.
RES = 22 # pin for hardware reset.
disp = LCDST7735(DC, RES)

start = True
led_on = False

disp.clearScreen()
disp.setCursor(0, 24)
disp.setTextColorBoth(WHITE)
disp.setTextSize(2)
disp.setTextWrap(False)
disp.drawString("Ultrasonic")
disp.fillRect2(0, 40, disp.WIDTH, 16, RED)

speaker.beepMessageReady()

def printLCD(number):
   disp.fillRect2(0, 40, disp.WIDTH, 16, RED)
   disp.setCursor(0, 40)
   disp.drawInt(number)

def avoid():
   vehicle.stop()
   distance = randint(5, 15)
   vehicle.backward(distance)
   time.sleep(0.5)
   angle = randint(0, 270) - 135
   time.sleep(0.5)
   vehicle.rotate(angle)
   disp.fillRect2(0, 40, disp.WIDTH, 16, RED)
   vehicle.forward()
   
try:

   while(True):
      if (start and button.isTouch()):
         start = False
         led.ledOff()
         speaker.beepMessageOk()
         time.sleep(3)
         vehicle.forward()
         vehicle.setSpeed(25)
         continue
      elif(start):
         led_on = not led_on
         if (led_on):
            led.ledOn()
         else:
            led.ledOff()
         time.sleep(0.5)
         continue
      elif ((not start) and button.isTouch()):
         vehicle.stop()
         start = True
         speaker.beepMessageOk()
         continue

      if (ussensor.distance() < 15.0):
         vehicle.stop()
         printLCD(math.floor(ussensor.distance()))
         avoid()

      if (bumper_left.isTouch() or bumper_right.isTouch()):
         avoid()

except KeyboardInterrupt:
   pass
finally:
   GPIO.cleanup()


