#!/usr/bin/python3
import tool4math as t4m

#################################################
# Picture Functions                             #
# file: tool4picture.py                         #
# Author: Th. Kaffka, Cologne, Germany          #
# Date: 10/17/2015                              #
# License: GNU General Public License (current) #
#################################################

errCode = 0
errMess = ""

###################################
# gets a list of the bitmap header.
def getBitmapHeader(img):
   offset = t4m.makeInt32(img[10], img[11], img[12], img[13])
   header_len = t4m.makeInt32(img[14], img[15], img[16], img[17])
   width = t4m.makeInt32(img[18], img[19], img[20], img[21])
   height = t4m.makeInt32(img[22], img[23], img[24], img[25])
   bit_count = t4m.makeInt16(img[28], img[29])
   compression = t4m.makeInt32(img[30], img[31], img[32], img[33])
   size_image = t4m.makeInt32(img[34], img[35], img[36], img[37])
   return (offset, header_len, width, height, bit_count, compression, size_image)

#######################
# makes a color string.
def makeColor(r, g, b):
    hex_number = "#"
    hex_digit = hex(r)[2:]
    if (len(hex_digit) == 1): hex_digit = "0" + hex_digit
    hex_number += hex_digit
    hex_digit = hex(g)[2:]
    if (len(hex_digit) == 1): hex_digit = "0" + hex_digit
    hex_number += hex_digit
    hex_digit = hex(b)[2:]
    if (len(hex_digit) == 1): hex_digit = "0" + hex_digit
    hex_number += hex_digit
    return hex_number

###########################
# gets a pixel of a bitmap.
def getPixel(row, col, img, header):
   offset = header[0]
   width = header[2]
   height = header[3]
   color_deep = header[4] // 8
   compression = header[5]
   width_corr = 0
   if (compression == 0):
      width_corr = width % 4
   x = offset + (row * width * color_deep) + (col * color_deep) + (width_corr * row)
   return (img[x], img[x+1], img[x+2])

####################################
# gets an average pixel of a bitmap.
def getPixelAverage(row, col, img, header, av):
   sum_pixel = [0, 0, 0]
   pixel_from = av // 2
   for i in range(0, av):
      pixel = getPixel(row - pixel_from + i, col, img, header)
      sum_pixel[0] += pixel[0]
      sum_pixel[1] += pixel[1]
      sum_pixel[2] += pixel[2]
   sum_pixel[0] /= av
   sum_pixel[1] /= av
   sum_pixel[2] /= av
   return sum_pixel

