#!/usr/bin/python
import sys
sys.path.append("/home/pi/Desktop/BrickPi_Python/Sensor_Examples")

from BrickPi import *
from threading import Thread
import time

########################################
# class lego motor                     #
# file: LegoMotor.py                   #
# Author: Th. Kaffka, Cologne, Germany #
# Date: 01/26/2016                     #
########################################

class LegoMotor(object):

   def __init__(self, port):
      self.port = port
      self.power = 0
      self.tacho_count = 0
      self.encoder = 0
      self.is_moving = False
      BrickPi.MotorEnable[port] = 1

   def __del__(self):
      pass
      
   def getPort(self):
      return self.port

   def getSpeed(self):
      return self.power

   def setSpeed(self, power):
      self.power = abs(power)

   def getEncoder(self):
      time.sleep(0.05)
      value = BrickPi.Encoder[self.port]
      return value
      
   def getEncoderDegrees(self):
      '''get the encoder as degrees'''
      pos = self.getEncoder() - self.encoder
      return (pos % 720) / 2

   def getTachoCount(self):
      return self.tacho_count
   
   def resetTachoCount(self):
      '''reset the tacho'''
      self.tacho_count = 0
      self.encoder = self.getEncoder()
      
   def countTacho(self):
      '''count the tacho'''
      self.tacho_count = abs(self.getEncoder() - self.encoder)
      
   def forward(self, fwd=0):
      '''move forward'''
      self.is_moving = True
      pos = self.getEncoder()
      BrickPi.MotorSpeed[self.port] = self.power
      if (not fwd == 0):
         self.t = Thread(target=self.doMove, args=(fwd,pos,1,))
         self.t.setDaemon(True)
         self.t.start()

   def backward(self, bwd=0):
      '''move back'''
      self.is_moving = True
      pos = self.getEncoder()
      BrickPi.MotorSpeed[self.port] = -self.power
      if (not bwd == 0):
         self.t = Thread(target=self.doMove, args=(bwd,pos,2,))
         self.t.setDaemon(True)
         self.t.start()

   def doMove(self, move, pos, direction):
      '''move the robot a distance in 0.5 degree'''
      count = 0
      if (direction == 1): #forward
         target = pos + move
         enc = self.getEncoder()
         while (pos < target):
            pos = self.getEncoder()
            count += 1
            if (count == 10): # avoid stalling.
               if (enc == self.getEncoder()):
                  print("stalling")
                  break
               else:
                  enc = self.getEncoder()
                  count = 0
            time.sleep(0.005)
         self.stop()
      else: #backward
         target = pos - move
         enc = self.getEncoder()
         while (pos > target):
            pos = self.getEncoder()
            count += 1
            if (count == 10): # avoid stalling.
               if (enc == self.getEncoder()):
                  print("stalling")
                  break
               else:
                  enc = self.getEncoder()
                  count = 0
            time.sleep(0.005)
         self.stop()

   def stop(self):
      '''stop the motor'''
      self.is_moving = False
      BrickPi.MotorSpeed[self.port] = 0

   def isMoving(self):
      return self.is_moving

   def rotate(self, angle):
      '''rotate a given angle (+, -)''' 
      self.is_moving = True
      self.t = Thread(target=self.doRotate, args=(angle,))
      self.t.setDaemon(True)
      self.t.start()

   def doRotate(self, angle):
      '''rotate a given angle (+, -)'''
      deg = self.getEncoder()
      target = deg + (angle * 2)
      if (angle > 0):
         self.forward()
         while (deg < target):
            deg = self.getEncoder()
         self.stop()
      elif (angle < 0):
         self.backward()
         while (deg > target):
            deg = self.getEncoder()
         self.stop()
      else:
         self.stop()

