#!/usr/bin/python
import sys
sys.path.append("/home/pi/Ae/Python/Interface/")
sys.path.append("/home/pi/Ae/Python/lib/")
import tool4file as t4f
from RPiCamera import RPiCamera

########################################
# class pattern recognition            #
# file: PatternRecogn.py               #
# Author: Th. Kaffka, Cologne, Germany #
# Date: 05/12/2016                     #
########################################

COLOR_RANGE = 6
PIXEL_STEP = 5

class PatternRecogn:

   def __init__(self, pic_width, pic_height):
      self.pic_width = pic_width
      self.pic_height = pic_height
      self.cam = RPiCamera()
      self.img_list = []
      self.init_list = []
      self.rect_list = []
      self.point_row_beg = -1
      self.point_col_beg = -1
      self.point_row_end = -1
      self.point_col_end = -1
      self.loadColorData()

   def __del__(self):
      del self.cam

   ######################
   # load the color data.
   def loadColorData(self):
      self.init_list = t4f.loadList("", "initcamera.txt")

   ####################
   # get all recangles.
   def getAllRect(self):
      self.makePicture()
      self.rect_list = []
      self.testArea()
      return self.rect_list

   ####################
   # get max. recangle.
   def getMaxRect(self):
      self.getAllRect()
      if (len(self.rect_list) > 0):
         ndx = self.maxRect()
         return self.rect_list[ndx]
      else:
         return (0, 0, 0, 0)

   #################
   # make a picture.
   def makePicture(self):
      self.img_list = self.cam.captureList(self.pic_width, self.pic_height)
      self.colorPicture()
      
   ##################
   # get the picture.
   def getPicture(self):
      return self.img_list

   ####################
   # color the picture.
   def colorPicture(self):
      length_init = len(self.init_list)
      length_img = len(self.img_list)
      for x in range(0, length_init):
         pixel = self.init_list[x]
         for y in range(0, length_img):
            img_pixel = self.img_list[y]
            if (self.testColor(img_pixel, pixel)):
               self.img_list[y] = (255, 255, 255)

   ################################
   # test the color of the picture.
   def testColor(self, img_pixel, pixel):
      if (img_pixel[0] >= pixel[0] - COLOR_RANGE):
         if (img_pixel[0] <= pixel[0] + COLOR_RANGE):
            if (img_pixel[1] >= pixel[1] - COLOR_RANGE):
               if (img_pixel[1] <= pixel[1] + COLOR_RANGE):
                  if (img_pixel[2] >= pixel[2] - COLOR_RANGE):
                     if (img_pixel[2] <= pixel[2] + COLOR_RANGE):
                        return True
      return False

   ################
   # test the area.
   def testArea(self):
      for row in range(1, self.pic_height, PIXEL_STEP):
         for col in range(1, self.pic_width):
            ndx = (row * self.pic_width) + col
            pixel = self.img_list[ndx]
            if (pixel == (255, 255, 255)):
               if (self.point_col_beg == -1):
                  self.point_row_beg = row
                  self.point_col_beg = col
                  self.identSecond(row, col)
                  new_rect = (self.point_col_beg, self.point_row_beg, self.point_col_end, self.point_row_end)
                  self.rect_list.append(new_rect)
                  self.point_col_beg = -1
                  row = self.point_row_end + 1
                  break

   ########################
   # identify second point.
   def identSecond(self, row_1, col_1):
      for col in range(col_1, self.pic_width):
         ndx = (row_1 * self.pic_width) + col
         pixel = self.img_list[ndx]
         if (not pixel == (255, 255, 255)):
            return
         for row in range(row_1, self.pic_height):
            ndx = (row * self.pic_width) + col
            pixel = self.img_list[ndx]
            if (not pixel == (255, 255, 255)):
               self.point_row_end = row
               self.point_col_end = col
               break
      self.point_row_end = self.pic_height
      self.point_col_end = self.pic_width

   ##############################
   # identify the max. rectangle.
   def maxRect(self):
      ndx = 0
      max_q = self.calcRect(self.rect_list[0])
      length = len(self.rect_list)
      for x in range(1, length):
         max_next = self.calcRect(self.rect_list[x])
         if (max_next > max_q):
            ndx = x
            max_q = max_next
      return ndx
         
   ###############################
   # calc the area of a rectangle.
   def calcRect(self, rect):
      x1 = rect[0]
      y1 = rect[1]
      x2 = rect[2]
      y2 = rect[3]
      return (x2 - x1) * (y2 - y1)
