#!/usr/bin/python
import sys
sys.path.append("/home/pi/Ae/Python/Interface")
sys.path.append("/home/pi/Desktop/BrickPi_Python/Sensor_Examples")
from BrickPi import *   								#import BrickPi.py file to use BrickPi operations
import time
from UpdateValues import *
import RPi.GPIO as GPIO

# import classes here.
from PatternRecogn import PatternRecogn
from Led import Led
from LegoMotor import LegoMotor
from LegoTouchSensor import LegoTouchSensor
from LegoColorSensor import LegoColorSensor
from HitechnicCompassSensor import HitechnicCompassSensor

########################################
# program can collector                #
# file: CanCollector.py                #
# Author: Th. Kaffka, Cologne, Germany #
# Date: 05/15/2016                     #
########################################

BrickPiSetup()  										# setup the serial port for communication

# global valibales.
PIC_WIDTH = 60       # width of camera picture.
PIC_HEIGHT = 44      # height of camera picture.
COLOR_BLACK = 1      # code of the color black.
COLOR_WHITE = 6      # code of the color white.
DIR_LEFT = 0         # code of left direction.
DIR_RIGHT = 1        # code of right direction.
FOUND_CAN = 0        # code of 'can found'.
FOUND_STOP = 1       # code of 'end status found'.
FOUND_NONE = 2       # code of 'found nothing'.

# parameter.
MOTOR_SPEED = 35     # the speed of the vehicle motors.
MOTOR_SPEED_SLOW = MOTOR_SPEED - 7 # the slow speed of the vehicle motors.
CLAW_SPEED = 110     # the speed of the claw motor.
GRIP_WIDTH = 90      # the width of the grip of the claw.
DIR_CORR = 7         # correction of the direction of the compass.
MID_CORR = 8         # correction of the picture mid.
CAN_DISTANCE = 29    # the distance of a can.

compass_straight = 0
compass_left = 0
compass_right = 0
compass_back = 0

# define the hardware.
p_recog = PatternRecogn(PIC_WIDTH, PIC_HEIGHT)
left_led = Led(12)
right_led = Led(13)
go = LegoTouchSensor(PORT_1, TYPE_SENSOR_TOUCH)
color = LegoColorSensor(PORT_2, TYPE_SENSOR_EV3_COLOR_M2)
compass = HitechnicCompassSensor(PORT_4, 0, 0)
motor_left = LegoMotor(PORT_D)
motor_right = LegoMotor(PORT_A)
motor_claw = LegoMotor(PORT_B)

BrickPiSetupSensors()									#Send the properties of sensors to BrickPi.  Set up the BrickPi.

compass.getDirection() # dummy, else error.
startUpdateValues()

#############################
# led signal: can identified.
def signalIdentify():
   left_led.ledOn();
   right_led.ledOn();
   time.sleep(0.1)
   left_led.ledOff();
   right_led.ledOff();
   time.sleep(0.1)
   
###########################################
# led signal: transfer to collecting point.
def signalTransfer():
   left_led.ledOn();
   right_led.ledOn();
   time.sleep(0.1)
   left_led.ledOff();
   right_led.ledOff();
   time.sleep(0.1)
   left_led.ledOn();
   right_led.ledOn();
   time.sleep(0.3)
   left_led.ledOff();
   right_led.ledOff();
   time.sleep(0.1)

#############################
# led signal: end collecting.
def signalEnd():
   left_led.ledOn();
   right_led.ledOn();
   time.sleep(0.1)
   left_led.ledOff();
   right_led.ledOff();
   time.sleep(0.1)
   left_led.ledOn();
   right_led.ledOn();
   time.sleep(0.3)
   left_led.ledOff();
   right_led.ledOff();
   time.sleep(0.1)
   left_led.ledOn();
   right_led.ledOn();
   time.sleep(0.1)
   left_led.ledOff();
   right_led.ledOff();
   time.sleep(0.1)

##############################################
# initalize the main directions of the compass.
def initDirection():
   global compass_straight, compass_left, compass_right, compass_back
   startPauseUpdate()
   time.sleep(0.02)
   compass_straight = compass.getDirection()
   endPauseUpdate()
   compass_left = corrDirection(compass_straight + 270)
   compass_right = corrDirection(compass_straight + 90)
   compass_back = corrDirection(compass_straight + 180)

###################################
# correcting the direction setting.
def corrDirection(direction):
   if (direction > 359):
      direction -= 360
   elif (direction < 0):
      direction += 360
   return direction

###############################################
# move forward, since a white bar ist detected.
def forwardSinceBar():
   motor_left.backward()
   motor_right.backward()
   while (True):
      time.sleep(0.04)
      light = color.getLight()
      if (light == COLOR_WHITE):
         break
   motor_left.stop()
   motor_right.stop()

################################################
# turn the vehicle, since target_dir is reached.
def turnSinceDir(direction, target_dir):
   if (direction == DIR_LEFT):
      motor_left.forward()
      motor_right.backward()
   elif (direction == DIR_RIGHT):
      motor_left.backward()
      motor_right.forward()
   while (not go.isTouched()):
      time.sleep(0.04)
      startPauseUpdate()
      time.sleep(0.02)
      curr_dir = compass.getDirection()
      endPauseUpdate()
      if (curr_dir - DIR_CORR <= target_dir and curr_dir + DIR_CORR >= target_dir):
         break
   motor_left.stop()
   motor_right.stop()

###########################
# grip a can with the claw.
def clawGrip():
   motor_claw.forward(GRIP_WIDTH)

##############################
# loose the can with the claw.
def clawUnGrip():
   motor_claw.backward(GRIP_WIDTH)

######################
# get the lowest rect.
def getLowRect():
   rect_list = p_recog.getAllRect()
   if (len(rect_list) > 0):
      ndx = lowRect(rect_list)
      return rect_list[ndx]
   else:
      return (0, 0, 0, 0)

################################
# identify the lowest rectangle.
def lowRect(rect_list):
   ndx = 0
   low_r = rect_list[0]
   length = len(rect_list)
   for x in range(1, length):
      low_next = rect_list[x]
      if (low_next[3] < low_r[3]):
         ndx = x
         low_r = low_next
   return ndx

############
# move back.
def moveBack(sec):
   motor_left.forward()
   motor_right.forward()
   end = time.time() + sec
   while (end > time.time()):
      time.sleep(0.04)
   motor_left.stop()
   motor_right.stop()

##########################################
# scanns the environment for cans or stop.
def scannCan():
   dir_left = corrDirection(compass_straight - 45)
   dir_right = corrDirection(compass_straight + 15)
   
   while (True):
      turnSinceDir(DIR_LEFT, dir_left)
      time.sleep(1.0)
      # turn right since can found or dir_right.
      motor_left.backward()
      motor_right.forward()
      while (True):
         time.sleep(0.04)
         startPauseUpdate()
         time.sleep(0.02)
         curr_dir = compass.getDirection()
         endPauseUpdate()
         if (curr_dir == -1):
            continue
         if (curr_dir - DIR_CORR <= dir_right and curr_dir + DIR_CORR >= dir_right):
            break
         rect = p_recog.getMaxRect()
         if (not rect == (0, 0, 0, 0)):
            motor_left.stop()
            motor_right.stop()
            time.sleep(1.0)
            return FOUND_CAN
      motor_left.stop()
      motor_right.stop()
      time.sleep(1.0)

      # move a bit forward.
      turnSinceDir(DIR_LEFT, compass_straight)
      motor_left.backward()
      motor_right.backward()
      end = time.time() + 2.5
      while (end > time.time()):
         time.sleep(0.04)
         light = color.getLight()
         if (light == COLOR_WHITE):
            motor_left.stop()
            motor_right.stop()
            time.sleep(1.0)
            return FOUND_STOP
      motor_left.stop()
      motor_right.stop()
      time.sleep(1.0)
   return FOUND_NONE

##################
# move to the can.
def goToCan():
   # center the can.
   motor_left.setSpeed(MOTOR_SPEED_SLOW)
   motor_right.setSpeed(MOTOR_SPEED_SLOW)
   rect = p_recog.getMaxRect()
   while (rect == (0, 0, 0, 0)):
      time.sleep(0.04)
      print("go to can: not found 1")
      motor_left.backward()
      motor_right.backward()
      time.sleep(0.3)
      motor_left.stop()
      motor_right.stop()
      rect = p_recog.getMaxRect()
   mid_rect = rect[0] + (rect[2] - rect[0]) / 2
   if (mid_rect < mid_pic):
      motor_left.forward()
      motor_right.backward()
   elif (mid_rect > mid_pic):
      motor_left.backward()
      motor_right.forward()
   while (not (mid_pic - MID_CORR < mid_rect and mid_pic + MID_CORR > mid_rect)):
      time.sleep(0.001)
      rect = p_recog.getMaxRect()
      if (rect == (0, 0, 0, 0)):
         print("go to can: not found 2")
         continue
      mid_rect = rect[0] + (rect[2] - rect[0]) / 2
   motor_left.stop()
   motor_right.stop()
   time.sleep(1.0)

   motor_left.setSpeed(MOTOR_SPEED)
   motor_right.setSpeed(MOTOR_SPEED)

   # go to the can.
   motor_left.backward()
   motor_right.backward()
   while (True):
      time.sleep(0.04)
      rect = getLowRect()
      if (rect[3] > CAN_DISTANCE):
         break
   motor_left.stop()
   motor_right.stop()
   time.sleep(1.0)

   # grap the can.
   motor_left.backward()
   motor_right.backward()
   end = time.time() + 2.5
   while (end > time.time()):
      time.sleep(0.04)
   motor_left.stop()
   motor_right.stop()
   time.sleep(1.0)

###########################
# move the can to the sink.
def moveToSink():
   clawGrip()
   time.sleep(1.0)

   turnSinceDir(DIR_RIGHT, compass_right)
   time.sleep(1.0)
   forwardSinceBar()
   time.sleep(1.0)

   turnSinceDir(DIR_RIGHT, compass_back)
   time.sleep(1.0)
   forwardSinceBar()
   clawUnGrip()
   time.sleep(1.0)
   
   moveBack(3.5)
   time.sleep(1.0)
   turnSinceDir(DIR_RIGHT, compass_straight)
   time.sleep(1.0)

try:

   initDirection()
   motor_left.setSpeed(MOTOR_SPEED)
   motor_right.setSpeed(MOTOR_SPEED)
   motor_claw.setSpeed(CLAW_SPEED)
#   motor_claw.forward(5000) # process this a the first time.
   mid_pic = (PIC_WIDTH / 2) - 7

   # start a collecting session.
   signalIdentify()
   while (not go.isTouched()):
      time.sleep(0.04)
   time.sleep(1.5)

   while (True):
      time.sleep(0.04)
      result = scannCan()
      if (result == FOUND_CAN):
         print ("can found")
         signalTransfer()
         goToCan()
         moveToSink()
      elif (result == FOUND_STOP):
         signalEnd()
         break

except KeyboardInterrupt:
   pass
finally:
   left_led.ledOff();
   right_led.ledOff();
   motor_left.stop()
   motor_right.stop()
   motor_claw.stop()
   endUpdateValues()
   GPIO.cleanup()
