#!/usr/bin/python3
import traceback
import glob
import os
import ast
from PIL import Image

########################################
# File Functions                       #
# file: tool4file.py                   #
# Author: Th. Kaffka, Cologne, Germany #
# Date: 10/17/2015                     #
########################################

errCode = 0
errMess = ""

######################
# loads a bitmap file.
def loadPicture(file_name):
   global errCode
   global errMess
   errCode = 0

   try:
      f = open ("./Bilder/" + file_name, "rb")
      img = f.read()
      f.close()
      return img
   except IOError:
      errCode = 1
      errMess = "File " + file_name + " not found!"
   except:
      print (traceback.print_exc())

#################################
# saves image to the file system.
def saveImage(path, file_name, image):
   try:
#      img = Image.fromarray(image)
#      img.save(path + file_name)
      image.save(path + file_name)
   except:
      print (traceback.print_exc())

###################################
# saves content to the file system.
def saveContent(path, file_name, content):
   try:
      f = open (path + file_name, "w")
      f.write(content)
      f.close()
   except:
      print (traceback.print_exc())

###################################
# loads content to the file system.
def loadContent(path, file_name):
   try:
      f = open (path + file_name, "r")
      content = f.read()
      f.close()
      return content
   except:
      print (traceback.print_exc())

###################################
# loads content to the file system.
def loadList(path, file_name):
   content = loadContent(path, file_name)
   return ast.literal_eval(content)

###################################################
# saves content in a file (name with pattern '@@').
def saveContentName(file_name, content):
   global errCode
   global errMess
   errCode = 0

   try:
      # generate the next file name.
      i = file_name.find("@@")
      if (i < 0):
         errCode = 2
         errMess = "save not successful, in file name no substring '@@' available."
         return
      
      length = len (file_name) - i
      file_count = 0
      file_list = glob.glob("../data/*.dat")
      for content_file in file_list:
         content_file = content_file.replace("../data/", "")
         content_file = content_file.replace("../data\\", "")
         count = int(content_file[i:-(length-2)])
         file_count = max (file_count, count)
      file_count = str(file_count + 1).zfill(2)
      file_name = file_name.replace("@@", file_count)
      
      f = open ("../data/" + file_name, "w")
      f.write(content)
      f.close()
   except:
      print (traceback.print_exc())

####################################
# delets all *.dat files in the dir.
def deletDatFiles():
   global errCode
   global errMess
   errCode = 0

   try:
      file_list = glob.glob("../data/*.dat")
      for content_file in file_list:
         os.remove (content_file)
   except:
      print (traceback.print_exc())
